<?php

namespace App\Http\Controllers;

use App\Models\FolderDirectoryModel;
use App\Models\User;
use App\Models\UserDirPermissionModel;
use Illuminate\Support\Facades\Validator;
use App\Service\HelperFunctionService;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Models\AuditLogModel;
use App\Models\SectionModel;


class AuditLogController extends Controller
{

    public const CODE = 'ALC' ;

    public function getLogs(Request $request){

        try{
            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;

            $log_data = AuditLogModel::select(
                'file_uid',
                'folder_id',
                'user_id',
                'action',
                'section_code',
                'log_type',
                'bussiness_id',
                'text',
                'created_at',
                'entity_type'
            )->where('bussiness_id',$bsns_id)
            ->orderBy('id','desc');

            if(isset($request->st_date) && isset($request->en_date) ){
                $log_data = $log_data->where('created_at','>=',$request->st_date.' 00:00:00')->where('created_at','<=',$request->en_date.' 23:59:59') ;
            }

            if( isset($request->user_id) ) {
                $log_data = $log_data->where('user_id',$request->user_id);
            }

            if( isset($request->action) ) {
                $log_data = $log_data->where('log_type',$request->action);
            }

            if( isset($request->entity_type) ) {
                $log_data = $log_data->where('entity_type',$request->entity_type);
            }

            if( isset($request->file_uid) ) {
                $log_data = $log_data->where('file_uid', 'like' , '%'. $request->file_uid.'%');
            }

            $log_data = $log_data->get()->toArray() ;

            $section_data = SectionModel::select('section_name','id','section_code')->where('bussiness_id',$bsns_id )
                ->get()->toArray() ;

            $section_map = [] ;
            foreach($section_data as $val){
                $section_map[$val['section_code']] = $val['section_name'] ;
            }

            $user_data = User::select('id','name')->where('bussiness_id',$bsns_id )
                ->whereNull('deleted_at')
                ->get()->toArray() ;

            $user_map = [] ;
            foreach($user_data as $val){
                $user_map[$val['id']] = $val['name'] ;
            }

            $result_ar = [] ;
            $i = 0 ;
            foreach($log_data as $val){


                $entity_t = '' ;

                if($val['entity_type'] == 1){
                    $entity_t = 'File' ;

                }else if($val['entity_type'] == 2){
                    $entity_t = 'Folder' ;
                }

                $section_name = '' ;
                if(isset($section_map[$val['section_code']])){
                    $section_name = $section_map[$val['section_code']] ;
                }else if( !$val['section_code'] ){
                    $section_name = '' ;
                }else{
                    $section_name = 'Common' ;
                }

                $result_ar[$i]['entity_type'] = $entity_t ;
                $result_ar[$i]['action'] = $val['action'] ;
                $result_ar[$i]['file_uid'] = $val['file_uid'] ;
                $result_ar[$i]['created_at'] = date('d M, Y | h:i:s a', strtotime( $val['created_at'])) ;
                $result_ar[$i]['section_name'] =  $section_name ;
                $result_ar[$i]['user_name'] = isset($user_map[$val['user_id']]) ? $user_map[$val['user_id']] : '-' ;
                $i++;
            }

            $user_ar = [] ;

            return response()->json([
                'status' => true,
                'data' => [
                    'logs' => $result_ar,
                    'user_ar' => $user_map
                ]
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


}
