<?php

namespace App\Http\Controllers;

use App\Models\User;
use Illuminate\Support\Facades\Validator;
use App\Service\HelperFunctionService;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Models\FolderDirectoryModel;
use App\Models\UserDirPermissionModel;
use App\Models\FileDocumentModel;
use App\Models\SectionModel;
use App\Service\MultiService;
use App\Service\AuditLogService;


class DeleteManagementController extends Controller
{

    public const CODE = 'DMC' ;

    # not permanent del
    public function delMultiple(Request $request){

        try {

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all() ;

            if( count($req) == 0 ){
                throw new \Exception('Invalid data.') ;
            }

            $bsns_id = $user_details->bussiness_id;

            $file_id_ar = [] ;
            $folder_id_ar = [] ;
            foreach($req as $val){

                if(!isset($val['id'])){
                    continue ;
                }

                if($val['is_folder'] == 1){

                    $is_upd = FolderDirectoryModel::where('id',$val['id'])
                    ->whereNull('deleted_at')
                    ->where('bussiness_id', $bsns_id)
                    ->update([
                        'deleted_at' => date('Y-m-d H:i:s')
                    ]);

                    UserDirPermissionModel::where('folder_id', $val['id'] )->where('bussiness_id', $bsns_id )
                    ->whereNull('deleted_at')
                    ->update([
                        'deleted_at' => date('Y-m-d H:i:s')
                    ]);

                    if($is_upd){
                        $folder_id_ar[] = $val['id'] ;
                    }

                    Log::info('FC001 :: folder del where folder id :  '. $val['id'] );

                }

                if($val['is_folder'] == 0){ # file

                    $is_upd = FileDocumentModel::where('unique_id',$val['id'])
                    ->whereNull('deleted_at')
                    ->where('bussiness_id', $bsns_id)
                    ->update([
                        'deleted_at' => date('Y-m-d H:i:s')
                    ]);

                    if($is_upd){
                        $file_id_ar[] = $val['id'] ;
                    }

                    Log::info('FC002 :: file del where file id :  '. $val['id'] );
                }
            }

            // audit log for file del start
            if(count($file_id_ar)){
                $file_data = FileDocumentModel::select( 'unique_id' , 'name' , 'folder_id' ,'section_code')
                ->whereNull('deleted_at')
                ->whereIn('unique_id', $file_id_ar)
                ->where('bussiness_id', $bsns_id)
                ->get()->toArray() ;

                foreach($file_data as $val){

                    $action_msg = "<b>File deleted permanent : </b>{$val['name']}";

                    AuditLogService::addAuditLog([
                        'file_uid' => $val['unique_id'],
                        'user_id' =>  $user_details->id,
                        'bussiness_id' => $user_details->bussiness_id,
                        'folder_id' => $val['folder_id'] ?? 0  ,
                        'action' => $action_msg,
                        'section_code' => $val['section_code'] ,
                        'entity_type' => 1,
                        'log_type' => 14 ,
                        'text' => 'file del' ,
                    ]) ;
                }
            }
            // audit log for file del end

            if(count($folder_id_ar)){

                $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')->whereIn('id', $folder_id_ar )
                ->get()->toArray() ;

                foreach($folder_data as $val){

                    AuditLogService::addAuditLog([
                        // 'file_uid' =>  $uniq_id,
                        'action' => '<b>Folder deleted : </b>'.$val['folder_name'] ,
                        'folder_id' =>  $val['id'],
                        'section_code' => $val['section_id'] ,
                        'bussiness_id' => $bsns_id ,
                        'user_id' =>  $user_details->id ,
                        'entity_type' => 2,
                        'log_type' => 15 ,
                        'text' => 'folder del' ,
                    ]) ;

                    $obj = new MultiService() ;
                    $obj->delRecursive(['folder_id'=>$val['id'], 'bussiness_id'=>$bsns_id ]);
                }
            }

            return response()->json([
                'status' => true,
                'message' => 'Items deleted successfully.'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function permDelSingle(Request $request){

        try{
            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;

            $req_ar =  $val = $request->all() ;

            if(!isset($req_ar['is_folder']) || !isset( $req_ar['id'] ) || !$req_ar['id'] || !$req_ar['is_folder'] ){
                throw new \Exception('Invalid Request Parameters') ;
            }

            if($req_ar['is_folder'] == 1){

                $is_upd = FolderDirectoryModel::where('id',$req_ar['id'])
                ->where('bussiness_id', $bsns_id)
                ->update([
                    'delete_level' => 1
                ]);

                Log::info('DM001 :: folder del permanent where folder id :  '. $req_ar['id']  );

                $obj = new MultiService() ;
                $obj->delPermRecursive(['folder_id'=>$req_ar['id'], 'bussiness_id'=>$bsns_id ]);
                Log::info('L009') ;

                if($is_upd){

                    $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')->where('id',$req_ar['id'])
                    ->get()->toArray() ;

                    foreach($folder_data as $val){

                        AuditLogService::addAuditLog([
                            // 'file_uid' =>  $uniq_id,
                            'action' =>  '<b>Folder deleted permanent : </b>'.$val['folder_name'] ,
                            'folder_id' =>  $val['id'],
                            'section_code' => $val['section_id'] ,
                            'bussiness_id' => $bsns_id ,
                            'user_id' =>  $user_details->id ,
                            'entity_type' => 2,
                            'log_type' => 23 ,
                            'text' => 'folder del permanent' ,
                        ]) ;

                    }
                }
            }


            if($req_ar['is_folder'] == 0){ # file

                $is_upd = FileDocumentModel::where('unique_id',$req_ar['id'])
                ->where('bussiness_id', $bsns_id)
                ->update([
                    'delete_level' => 1
                ]);

                Log::info('DM002 :: file del permanent where file id :  '. $req_ar['id'] );

                // audit log for file del start
                if($is_upd){
                    $file_data = FileDocumentModel::select( 'unique_id' , 'name' , 'folder_id' ,'section_code')
                    ->where('unique_id',$req_ar['id'])
                    ->where('bussiness_id', $bsns_id)
                    ->get()->toArray() ;

                    foreach($file_data as $val){

                        $action_msg = "<b>File deleted permanent : </b>{$val['name']}";

                        AuditLogService::addAuditLog([
                            'file_uid' => $val['unique_id'],
                            'user_id' =>  $user_details->id,
                            'bussiness_id' => $user_details->bussiness_id,
                            'folder_id' => $val['folder_id'] ?? 0  ,
                            'action' => $action_msg,
                            'section_code' => $val['section_code'] ,
                            'entity_type' => 1,
                            'log_type' => 22 ,
                            'text' => 'file del permanent' ,
                        ]) ;
                    }
                }
                // audit log for file del end
            }

            return response()->json([
                'status' => true,
                'data' => 'Item Deleted permanently.'
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'02',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                // 'message'=> $e->getMessage()
                'message'=> $err
            ],200);
       }
    }

    public function permDelMultiple(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all() ;

            if( count($req) == 0 ){
                throw new \Exception('Invalid data.') ;
            }

            $bsns_id = $user_details->bussiness_id;

            $file_id_ar = [] ;
            $folder_id_ar = [] ;
            foreach($req as $val){

                if(!isset($val['id'])){
                    continue ;
                }

                if($val['is_folder'] == 1){

                    $is_upd = FolderDirectoryModel::where('id',$val['id'])
                    ->where('bussiness_id', $bsns_id)
                    ->update([
                        'delete_level' => 1
                    ]);

                    UserDirPermissionModel::where('folder_id', $val['id'] )->where('bussiness_id', $bsns_id )
                    ->update([
                        'delete_level' => 1
                    ]);

                    if($is_upd){
                        $folder_id_ar[] = $val['id'] ;
                    }

                    Log::info('DM007 :: folder del permanent where folder id :  '. $val['id'] );

                }

                if($val['is_folder'] == 0){ # file

                    $is_upd = FileDocumentModel::where('unique_id',$val['id'])
                    ->where('bussiness_id', $bsns_id)
                    ->update([
                        'delete_level' => 1
                    ]);

                    if($is_upd){
                        $file_id_ar[] = $val['id'] ;
                    }

                    Log::info('DM008 :: file del permanent where file id :  '. $val['id'] );
                }
            }

              // audit log for file del start
              if(count($file_id_ar)){
                $file_data = FileDocumentModel::select( 'unique_id' , 'name' , 'folder_id' ,'section_code')
                ->whereIn('unique_id', $file_id_ar)
                ->where('bussiness_id', $bsns_id)
                ->get()->toArray() ;

                foreach($file_data as $val){

                    $action_msg = "<b>File deleted permanent : </b>{$val['name']}";

                    AuditLogService::addAuditLog([
                        'file_uid' => $val['unique_id'],
                        'user_id' =>  $user_details->id,
                        'bussiness_id' => $user_details->bussiness_id,
                        'folder_id' => $val['folder_id'] ?? 0  ,
                        'action' => $action_msg,
                        'section_code' => $val['section_code'] ,
                        'entity_type' => 1,
                        'log_type' => 22 ,
                        'text' => 'file del permanent' ,
                    ]) ;
                }
            }
            // audit log for file del end

            if(count($folder_id_ar)){

                $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')->whereIn('id', $folder_id_ar )
                ->get()->toArray() ;

                foreach($folder_data as $val){

                    AuditLogService::addAuditLog([
                        // 'file_uid' =>  $uniq_id,
                        'action' =>  '<b>Folder deleted permanent : </b>'.$val['folder_name'] ,
                        'folder_id' =>  $val['id'],
                        'section_code' => $val['section_id'] ,
                        'bussiness_id' => $bsns_id ,
                        'user_id' =>  $user_details->id ,
                        'entity_type' => 2,
                        'log_type' => 23 ,
                        'text' => 'folder del permanent' ,
                    ]) ;

                    $obj = new MultiService() ;
                    $obj->delPermRecursive(['folder_id'=>$val['id'], 'bussiness_id'=>$bsns_id ]);
                }
            }

            return response()->json([
                'status' => true,
                'message' => 'Items deleted successfully.'
            ], 200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'03',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
        }
    }


}
