<?php

namespace App\Http\Controllers;

use App\Models\GroupModel;
use App\Models\GroupUserAssocModel;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use App\Service\HelperFunctionService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class GroupController extends Controller
{

    public const CODE = 'G' ;

    public function groupList(Request $request){

        try{

            $group_details = Auth::user();

            if(!$group_details){
                throw new \Exception('User is not authenticate');
            }

            if($group_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'user_id'=>'nullable|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $group_details->bussiness_id ;

            $group_d = GroupModel::select()->where('bussiness_id',$bsns_id)->where('added_by',$group_details->id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            foreach($group_d as $k=>$val){
                $group_d[$k]['created_at'] = date('M d,Y', strtotime($val['created_at']) );
            }

            return response()->json([
                'status' => true,
                'data' => $group_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    /**
     * add and edit group
     */
    public function addGroup( Request $request){

        try{

            $group_details = Auth::user();

            if(!$group_details){
                throw new \Exception('User is not authenticate');
            }

            if($group_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                // 'group_name'=>'required|regex:/^[a-zA-Z0-9\s]*$/', // alpha numeric with space are allowed
                'group_name'=>'required|regex:/^[a-zA-Z0-9 -]*$/', // alpha numeric with space and hypen are allowed
                'is_edit' => 'required',
                'group_id' => 'nullable',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            if($req['is_edit']){
                GroupModel::where('id',$req['group_id'])->update([
                    'group_name' => $req['group_name'],
                ]);

                return response()->json([
                    'status' => true,
                    'message' => 'Group updated successfully.',
                ],200);
            }

            $bsns_id = $group_details->bussiness_id ;

            $sec_id = GroupModel::create([
                'group_name' => $req['group_name'],
                'bussiness_id' => $bsns_id,
                'added_by' => $group_details->id,
            ])->id;

            return response()->json([
                'status' => true,
                'message' => 'Group added successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'02',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function deleteGroup(Request $request){

        try{

            $group_details = Auth::user();

            if(!$group_details){
                throw new \Exception('User is not authenticate');
            }

            if($group_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'group_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            GroupModel::where('id',$req['group_id'])->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Group_id Deleted successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'03',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function getGroupAssocUser(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'group_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $user_data = User::select('id','name')->where('bussiness_id',$bsns_id)->where('id','!=',$user_details->id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            $group_assoc = GroupUserAssocModel::select('id','user_id')->where('bussiness_id',$bsns_id)->where('group_id',$req['group_id'])->get()->toArray();

            $user_id_associated = [];
            foreach($group_assoc as $val){
                $user_id_associated[] = $val['user_id'] ;
            }

            $_data = [] ;

            foreach($user_data as  $k=>$val){
                $_data[$k]['user_id'] = $val['id'] ;
                $_data[$k]['user_name'] = $val['name'] ;
                $_data[$k]['is_associated'] = false ;

                if(in_array( $val['id'] ,$user_id_associated ) ){
                    $_data[$k]['is_associated'] = true ;
                }
            }

            return response()->json([
                'status' => true,
                'data' => $_data,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'07',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function saveGroupAssocUser(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'group_id'=>'required|numeric',
                'user_id_ar'=>'array|nullable'
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $user_id_ar = $request->user_id_ar ;

            $bsns_id = $user_details->bussiness_id ;

            GroupUserAssocModel::where('group_id',$request->group_id)->where('bussiness_id',$bsns_id )->delete();

            foreach($user_id_ar as $val){
                GroupUserAssocModel::create([
                    'group_id' => $request->group_id,
                    'user_id' => $val,
                    'bussiness_id' => $bsns_id
                ]) ;
            }

            return response()->json([
                'status' => true,
                'message' => 'Data updated successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'07',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function getGroupFolderAssoc(Request $request){

        try{


        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'08',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
        }
    }

    public function addFolderUGAssoc(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'group_id'=>'required',
                'folder_id'=>'required|numeric'
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validator = $validation->validated();

            // echo '<pre>';
            // print_r($validator);
            // exit ;

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'09',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
        }
    }

}
