<?php

namespace App\Http\Controllers;

use App\Models\GroupModel;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use App\Models\FileDocumentModel;
use App\Models\FolderDirectoryModel;
use App\Models\SectionModel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Service\FileDetailService;
use App\Service\MultiService;

class RecoveryController extends Controller
{

    public const CODE = 'RC' ;

    public function getDeletedItems(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if( $user_details->role_id!=2 ){
                throw new \Exception('Permission denied');
            }

            $section_data = SectionModel::select('section_name','id','section_code')
            ->where('bussiness_id', $user_details->id )
            ->get()->toArray() ;

            $section_map = [] ;
            foreach($section_data as $val){
                $section_map[$val['section_code']] = $val['section_name'] ;
            }

            $document_ar = FileDocumentModel::select('id','name','section_code','folder_id','unique_id','updated_at')
                ->whereNotNull('deleted_at')
                ->orderBy('updated_at', 'asc')
                ->where('delete_level',0)
                ->where('bussiness_id', $user_details->id )
                ->get()->toArray() ;

            $folder_ar = FolderDirectoryModel::select(
                'id',
                'folder_name',
                'section_id',
                'parent_folder_id',
                'added_by',
                'updated_at'
            )
                ->whereNotNull('deleted_at')
                ->where('delete_level',0)
                ->orderBy('updated_at', 'asc')
                ->where('bussiness_id', $user_details->id )
                ->get()->toArray() ;

            $_resp = [];
            $i = 0 ;
            foreach($folder_ar as $val){
                $_resp[$i]['is_folder'] = 1;
                $_resp[$i]['folder_id'] = $val['id'];
                $_resp[$i]['section_name'] = isset($section_map[$val['section_id']]) ? $section_map[$val['section_id']] : 'Common' ;
                $_resp[$i]['name'] = $val['folder_name'];
                $_resp[$i]['deleted_at'] =  date('d M, Y h:i:s a', strtotime($val['updated_at']));
                $i++;
            }

            foreach($document_ar as $val){
                $_resp[$i]['is_folder'] = 0;
                $_resp[$i]['section_name'] = isset($section_map[$val['section_code']]) ? $section_map[$val['section_code']] : 'Common' ;
                $_resp[$i]['name'] = $val['name'];
                $_resp[$i]['unique_id'] = $val['unique_id'];
                $_resp[$i]['deleted_at'] =  date('d M, Y h:i:s a', strtotime($val['updated_at']));
                $i++;
            }

            return response()->json([
                'status' => true,
                'data' => $_resp
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $err
            ],200);
       }
    }

    // audit log pending for recovery
    public function recoverItem(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if( $user_details->role_id!=2 ){
                throw new \Exception('Permission denied');
            }

            $validation = Validator::make($request->all(),[
                'is_folder' => 'required',
                'id' => 'required'
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            $bsns_id = $user_details->bussiness_id;

            if( $validated['is_folder'] !=  1){ // for file
                $msg = 'Document recoverd successfully.' ;

                $document_ar = FileDocumentModel::select('id','name','section_code','folder_id','unique_id','updated_at')
                ->where('unique_id', $validated['id'] )
                ->where('bussiness_id', $bsns_id)
                ->first() ;

                if(!$document_ar){
                    throw new \Exception('Invalid Document') ;
                }

                if($document_ar->folder_id != $document_ar->section_code ){

                    $res = FileDetailService::getFilePath($document_ar->folder_id, $document_ar->section_code , null ) ;
                    unset($res['folder_ids'][0]) ;
                    $folder_ids = array_values( $res['folder_ids'] );

                    FolderDirectoryModel::whereIn('id', $folder_ids )
                    ->where('delete_level', 0)
                    ->update( ['deleted_at'=>null] );
                }

                FileDocumentModel::where('unique_id', $validated['id'] )
                ->where('bussiness_id', $bsns_id)
                ->update( ['deleted_at'=>null] );

            }else{ // for folder 
                $msg = 'Folder recoverd successfully.' ;

                FolderDirectoryModel::where('id', $validated['id'] )
                ->where('delete_level', 0)
                ->where('bussiness_id', $bsns_id)
                ->update( ['deleted_at'=>null] );

                $obj = new MultiService() ;
                $obj->recoverRecursive(['folder_id'=> $validated['id'] , 'bussiness_id'=>$bsns_id ]);

                //--> now we need the permanent delt
                // FileDocumentModel::where('id', $validated['id'] )
                // ->update( ['deleted_at'=>null] );

            }

            return response()->json([
                'status' => true,
                'message' => $msg
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'02',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $err
            ],200);
       }
    }
}
