<?php

namespace App\Http\Controllers;

use App\Models\RoleModel;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;
use App\Models\GroupModel;
use Illuminate\Support\Facades\Log;


class RoleController extends Controller
{
    public const CODE = 'RC' ;
    public function store(Request $request)
    {
        try{
            $input = $request->all();
            $validation = Validator::make($input,[
                'name'=>"required",
            ]);
            if($validation->fails()){
                return response()->json([
                    'status'=>false ,
                    'errors'=>$validation->errors()
                ],200);
            }
            else{
                $role_model= new RoleModel;
                $role_model->name = $request->name;
                $res=$role_model->save();
                if($res){
                    return response()->json([
                        'status'=>200,
                        'message'=>"Role Created Successfuly"
                    ],200);
                }
                else{
                    return response()->json([
                        'status'=>500,
                        'message'=>"Somethig went wrong"
                    ],500);
                }
            }
        }catch(\Throwable $e){

            $err = [
                'code' => self::CODE.'03',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>500,
                'err' => $err,

            ],500);

        }

    }

    public function show($id)
    {
        try{
            if(!$id){
                throw new \Exception('id not found') ;
            }
            $data = RoleModel::select('id',
            'name')->where('id',$id)->first();
            if($data){
                return response()->json([
                    'status'=>200,
                    'data'=>$data,
                    'message'=>'role details'

                ],200);
            }else{
                return response()->json([
                    'status'=>402,
                    'message'=>'id not found !!!'

                ],402);
            }
        }catch(\Throwable $e){

            $err = [
                'code' => self::CODE.'02',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>500,
                'err' => $err,

            ],500);

        }
    }

    public function update(Request $request)
    {
        try{
            $input = $request->all();

            $validation = Validator::make($input,[
                'id'=>'required|string',
                'name'=>'required|string',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'errors'=>$validation->errors()
                ],200);
            }
            else{
                $data_to_update=[];
                $role_model= new RoleModel;
                $data_to_update['name']=$request->name;
                $res=RoleModel::where('id',$request->id)->update($data_to_update);
                if($res){
                    return response()->json([
                        'status'=>200,
                        'message'=>'Role Updated !!!'

                    ],200);
                }else{
                    return response()->json([
                        'status'=>500,
                        'message'=>'Something went wrong !!!'

                    ],500);
                }
            }
        }catch(\Throwable $e){

            $err = [
                'code' => self::CODE.'04',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>500,
                'err' => $err,
            ],500);
        }
    }

    public function del(Request $request){
        try{
            $id=$request->id;
            if(!$id){
                throw new \Exception('id not Found');
            }

            $res=RoleModel::where('id',$id)->delete();
            if($res){
                return response()->json([
                    'status'=>200,
                    'message'=>'Role Deleted !!!'

                ],200);
            }else{
                return response()->json([
                    'status'=>500,
                    'message'=>'id not exist !!!'

                ],500);
            }
        }catch(\Throwable $e){

            $err = [
                'code' => self::CODE.'05',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>500,
                'err' => $err,

            ],500);

        }
    }
}
