<?php

namespace App\Http\Controllers;

use App\Models\SectionModel;
use App\Models\SectionUserAssocModel;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use App\Models\UserDirPermissionModel;
use App\Service\HelperFunctionService;
use App\Service\SectionService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class SectionController extends Controller
{

    public const CODE = 'S' ;

    public function sectionList(Request $request){

        try{

            $section_details = Auth::user();

            if(!$section_details){
                throw new \Exception('User is not authenticate');
            }

            if($section_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'user_id'=>'nullable|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $section_details->bussiness_id ;

            $section_d = SectionModel::select()->where('bussiness_id',$bsns_id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            foreach($section_d as $k=>$val){
                $section_d[$k]['created_at'] = date('M d,Y', strtotime($val['created_at']) );
            }

            return response()->json([
                'status' => true,
                'data' => $section_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function sectionListNav(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if(!$user_details->role_id){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $bsns_id = $user_details->bussiness_id ;

            $section_d = [];
            if($user_details->role_id==2){

                $section_d = SectionModel::select('section_name','id','section_code')->where('bussiness_id',$bsns_id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            }else if($user_details->role_id==3){

                $section_d = SectionService::getUserNavigation($user_details) ;
            }

            return response()->json([
                'status' => true,
                'data' => $section_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    /**
     * add and edit section
     */
    public function addSection( Request $request){

        try{

            $section_details = Auth::user();

            if(!$section_details){
                throw new \Exception('User is not authenticate');
            }

            if($section_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                // 'section_name'=>'required|regex:/^[a-zA-Z0-9\s]*$/', // alpha numeric with space are allowed
                'section_name'=>'required|regex:/^[a-zA-Z0-9 -]*$/', // alpha numeric with space and hypen are allowed
                'is_edit' => 'required',
                'section_id' => 'nullable',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            if($req['is_edit']){
                SectionModel::where('id',$req['section_id'])->update([
                    'section_name' => $req['section_name'],
                ]);

                return response()->json([
                    'status' => true,
                    'message' => 'Section updated successfully.',
                ],200);
            }

            $bsns_id = $section_details->bussiness_id ;

            $sec_id = SectionModel::create([
                'section_name' => $req['section_name'],
                'bussiness_id' => $bsns_id,
                'added_by' => $section_details->id,
            ])->id;

            $code = HelperFunctionService::getCode( 'S', $sec_id ) ;

            SectionModel::where('id',$sec_id )->update([
                'section_code' => $code
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Section added successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'02',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function deleteSection(Request $request){

        try{

            $section_details = Auth::user();

            if(!$section_details){
                throw new \Exception('User is not authenticate');
            }

            if($section_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'section_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            SectionModel::where('id',$req['section_id'])->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Section Deleted successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'03',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function sectionDetail(Request $request){

        try{

            $section_details = Auth::user();

            if(!$section_details){
                throw new \Exception('User is not authenticate');
            }

            if($section_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'section_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $section_d = SectionModel::select('section_name','id')->where('id',$req['section_id'])->whereNull('deleted_at')->first();

            if(!$section_d){
                throw new \Exception('Section not found') ;
            }

            return response()->json([
                'status' => true,
                'data' => $section_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'06',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function getSectionAssocUser(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'section_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $user_data = User::select('id','name')->where('bussiness_id',$bsns_id)->where('id','!=',$user_details->id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            $section_assoc = SectionUserAssocModel::select('id','user_id')->where('bussiness_id',$bsns_id)->where('section_id',$req['section_id'])->get()->toArray();

            $user_id_associated = [];
            foreach($section_assoc as $val){
                $user_id_associated[] = $val['user_id'] ;
            }

            $_data = [] ;

            foreach($user_data as  $k=>$val){
                $_data[$k]['user_id'] = $val['id'] ;
                $_data[$k]['user_name'] = $val['name'] ;
                $_data[$k]['is_associated'] = false ;

                if(in_array( $val['id'] ,$user_id_associated ) ){
                    $_data[$k]['is_associated'] = true ;
                }
            }

            return response()->json([
                'status' => true,
                'data' => $_data,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'07',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function saveSectionAssocUser(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'section_id'=>'required|numeric',
                'user_id_ar'=>'array|nullable'
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $user_id_ar = $request->user_id_ar ;

            $bsns_id = $user_details->bussiness_id ;

            SectionUserAssocModel::where('section_id',$request->section_id)->where('bussiness_id',$bsns_id )->delete();

            foreach($user_id_ar as $val){
                SectionUserAssocModel::create([
                    'section_id' => $request->section_id,
                    'user_id' => $val,
                    'bussiness_id' => $bsns_id
                ]) ;
            }

            return response()->json([
                'status' => true,
                'message' => 'Data updated successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'07',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

}
