<?php

namespace App\Http\Controllers;

use App\Models\FolderDirectoryModel;
use App\Models\User;
use App\Models\UserDirPermissionModel;
use Illuminate\Support\Facades\Validator;
use App\Service\HelperFunctionService;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Service\NotificationService;

class UserDirPermission extends Controller
{

    public const CODE = 'UDP' ;

    public function userListForDirPermission( Request $request){

        try{
            $user_details = HelperFunctionService::doValidateForUser() ;

            $bsns_id = $user_details->bussiness_id ;

            $user_d = User::select('id','name', 'email')->where('bussiness_id',$bsns_id)->where('id','!=',$user_details->id)
            ->where('role_id',3)
            ->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            return response()->json([
                'status' => true,
                'data' => $user_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }
    public function getUserPerm( Request $request){

        try{
            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'folder_id'=>  [
                    'required',
                    'numeric',
                    Rule::notIn([0]),
                ],
                'user_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            $bsns_id = $user_details->bussiness_id ;

            $user_dir_perm = UserDirPermissionModel::select('id', 'view',
            'edit','download','delete',)->where('bussiness_id',$bsns_id)->where('assigned_to',$validated['user_id'])
            ->where('folder_id',$validated['folder_id'])
            ->whereNull('deleted_at')->first() ;

            return response()->json([
                'status' => true,
                'data' => $user_dir_perm,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'03',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function saveUserPerm( Request $request){

        try{
            $user_details = HelperFunctionService::doValidateForUser() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'folder_id'=>  [
                    'required',
                    'numeric',
                    Rule::notIn([0]),
                ],
                'user_id'=>  [
                    'required',
                    'numeric',
                    Rule::notIn([0]),
                ],
                'view_perm'=>  [
                    'required',
                    'numeric',
                    Rule::in([0,1]),
                ],
                'edit_perm'=>  [
                    'required',
                    'numeric',
                    Rule::in([0,1]),
                ],
                'download_perm'=>  [
                    'required',
                    'numeric',
                    Rule::in([0,1]),
                ],
                'delete_perm'=>  [
                    'required',
                    'numeric',
                    Rule::in([0,1]),
                ],
                'folder_name' => 'string|required' ,
                'section_name' => 'string|required'
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            // echo 1900;


            $validated = $validation->validated();

            $bsns_id = $user_details->bussiness_id ;

            $user_dir_perm = UserDirPermissionModel::select('id', 'view',
            'edit','download','delete',)->where('bussiness_id',$bsns_id)->where('assigned_to',$validated['user_id'])
            ->where('folder_id',$validated['folder_id'])
            ->whereNull('deleted_at')->first() ;

            $section_code = FolderDirectoryModel::select('id','section_id')->where( 'id', $validated['folder_id']  )->whereNull('deleted_at')->first() ;

            if(!$section_code){
                throw new \Exception('Folder not valid') ;
            }

            if($user_dir_perm == null){
                UserDirPermissionModel::create([
                    'section_code' => $section_code->section_id ,
                    'bussiness_id' => $bsns_id,
                    'added_by' => $user_details->id,
                    'folder_id' => $validated['folder_id'] ,
                    'assigned_to' => $validated['user_id'],
                    'view' => $validated['view_perm'],
                    'edit' => $validated['edit_perm'],
                    'download' => $validated['download_perm'],
                    'delete' => $validated['delete_perm'],
                ]);
            }else{
                UserDirPermissionModel::where('id',$user_dir_perm->id)
                ->where('bussiness_id',$bsns_id)
                ->update([
                    'section_code' => $section_code->section_id ,
                    'added_by' => $user_details->id,
                    'folder_id' => $validated['folder_id'] ,
                    'assigned_to' => $validated['user_id'],
                    'view' => $validated['view_perm'],
                    'edit' => $validated['edit_perm'],
                    'download' => $validated['download_perm'],
                    'delete' => $validated['delete_perm'],
                ]);
            }


            // notification start

            $desc =  $user_details->name . " granted you access on Folder : {$req['folder_name']} , Section : {$req['section_name']} " ;

            NotificationService::addNotification([
                'title' => 'Folder Permission Given',
                'description' => $desc,
                'notification_type' => 1, // permission given to folder
                'bussiness_id' => $user_details->bussiness_id,
                'source_user_id' => $user_details->id,
                'destination_user_id' => $req['user_id'],
                'folder_id' =>  $validated['folder_id'],
                'section_code' => $section_code->section_id ,
            ]);
            // notification end

            return response()->json([
                'status' => true,
                'message' => "Permission Updated Successfully"
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'04',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
        }

    }


}
