<?php

namespace App\Service;

use App\Models\FileDocumentModel;
use App\Models\BussinessModel;
use App\Models\SectionModel;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use Illuminate\Support\Facades\DB;



class DashboardService
{
    //! if folder delt space should not count
    //! if section delt space should not count
    public static function getAvailStorage($user_details){

        $result = FileDocumentModel::selectRaw('SUM(size_1) as size_1')->whereNull('deleted_at')
        ->where('bussiness_id',$user_details->bussiness_id )->first();

        $res = ( $result->size_1 / 1024 ) / 1024 ; // into GB
        $res = round($res, 2) ;

        $bsns_data = BussinessModel::select('current_plan','plan_expired_on','created_at')->where('id',$user_details->bussiness_id )->first() ;

        if(!$bsns_data){
            throw new \Exception('Plan not found') ;
        }

        $current_plan = $bsns_data->current_plan ;

        $plan_ar = config('plan.plan.plan_type') ;

        if(!$current_plan || !isset($plan_ar[$current_plan]) ){
            throw new \Exception('Invalid Plan') ;
        }

        $alloted_size = $plan_ar[$current_plan]['storage']   ;

        // check plan expiry start
        $plan_expired_on = $bsns_data->plan_expired_on ?? null ;
        $rem_days = null ;

        if($plan_expired_on){

            $timestamp1 = strtotime( date('Y-m-d') );
            $timestamp2 = strtotime( $plan_expired_on );

            if($timestamp1 > $timestamp2 ){ // plan is already expired
                $rem_days = -1 ;
            }else{
                $diffInSeconds = $timestamp2 - $timestamp1;
                // Calculate the difference in days
                $rem_days = floor($diffInSeconds / (60 * 60 * 24));
            }
        }
        // check plan expiry end

        return [
            'status' => true ,
            'data' => [
                'used' => $res, // in GB
                'free' =>  round ($alloted_size - $res , 2 ) // in mb
            ] ,
            'total_users' =>  $plan_ar[$current_plan]['users'] ,
            'rem_days' =>  $rem_days
        ];
    }

    public static function totalUploadedDocs($user_details){

        $result = FileDocumentModel::whereNull('deleted_at')
        ->where('bussiness_id',$user_details->bussiness_id )->count();

        $user_res = User::whereNull('deleted_at')
        ->where('role_id',3)
        ->where('bussiness_id',$user_details->bussiness_id )->count();

        return [
            'status' => true ,
            'total_uploads' => $result,
            'total_users' => $user_res
        ];
    }

    public static function sectionSpaceConsumption($user_details){

        $section_data = SectionModel::select('section_name','section_code')
            ->whereNull('deleted_at')
            ->where('bussiness_id',$user_details->bussiness_id )
            ->get()->toArray();


        if( count($section_data)==0 ){
            return [
                'status' => true ,
                'space' => []
            ] ;
        }

        $sec_map = [];
        foreach($section_data as $v ){
            $sec_map[$v['section_code']] = $v['section_name'] ;
        }


        $result = FileDocumentModel::selectRaw('SUM(size_1) as size_1 , section_code')
            ->whereNull('deleted_at')
            ->groupBy('section_code')
            ->where('bussiness_id',$user_details->bussiness_id )->get()->toArray();

        $output = [] ;

        foreach($result as $v ){

            if(isset( $sec_map[$v['section_code']] ) ){

                $sec_name = $sec_map[$v['section_code']] ;
                $spc = round ( ( $v['size_1'] / 1024 ), 2) ; // divd by 1024  ; // into mb

                $output[] = [
                    'name' => "$sec_name" ,
                    'y' => $spc
                ] ;
            }
        }

        return [
            'status' => true ,
            'space' => $output
        ] ;

    }


    public static function docFormat($user_details){

        $currentDate = date('Y-m-d');
        $firstDayOfMonth = date('Y-m-01', strtotime($currentDate));
        $lastDayOfMonth = date('Y-m-t', strtotime($currentDate));

        $excel = [ 'xlsx','xls','xlsm','xlsb','xltx','xltm','csv','ods','xps' ] ;

        $image_extensions = ['jpg', 'jpeg', 'png', 'gif', 'bmp', 'tiff', 'tif', 'raw', 'svg', 'webp', 'pdf', 'eps', 'ico', 'tga', 'psd', 'exif', 'heif', 'heic'] ;

        $pdf_extensions = ['pdf', 'pdfa', 'pdfx', 'pdfvt'] ;

        $res_excel_cnt = FileDocumentModel::selectRaw('SUM(size_1) as size_1 , section_code')
            ->whereNull('deleted_at')
            ->whereIn('extension',$excel )
            ->where('created_at','>=', $firstDayOfMonth)
            ->where('created_at','<=', $lastDayOfMonth)
            ->where('bussiness_id',$user_details->bussiness_id )->count();

        $res_img_cnt = FileDocumentModel::selectRaw('SUM(size_1) as size_1 , section_code')
            ->whereNull('deleted_at')
            ->whereIn('extension',$image_extensions )
            ->where('created_at','>=', $firstDayOfMonth)
            ->where('created_at','<=', $lastDayOfMonth)
            ->where('bussiness_id',$user_details->bussiness_id )->count();

        $res_pdf_cnt = FileDocumentModel::selectRaw('SUM(size_1) as size_1 , section_code')
            ->whereNull('deleted_at')
            ->whereIn('extension',$pdf_extensions )
            ->where('created_at','>=', $firstDayOfMonth)
            ->where('created_at','<=', $lastDayOfMonth)
            ->where('bussiness_id',$user_details->bussiness_id )->count();

        $total_this_month_upload = FileDocumentModel::selectRaw('SUM(size_1) as size_1 , section_code')
            ->whereNull('deleted_at')
            ->where('created_at','>=', $firstDayOfMonth)
            ->where('created_at','<=', $lastDayOfMonth)
            ->where('bussiness_id',$user_details->bussiness_id )->count();


        $other_cnt = $total_this_month_upload - ($res_excel_cnt +  $res_img_cnt +  $res_pdf_cnt) ;

        return [
            [
                'name' => "Excel" ,
                'y' => $res_excel_cnt
            ] ,
            [
                'name' => "Image" ,
                'y' => $res_img_cnt
            ] ,
            [
                'name' => "Pdf" ,
                'y' => $res_pdf_cnt
            ] ,
            [
                'name' => "Other" ,
                'y' => $other_cnt
            ]
        ];
    }

    public static function monthWiseUploads($user_details){

        $curent_year = date('Y');

        $results = FileDocumentModel::select(
            DB::raw('YEAR(created_at) as upload_year'),
            DB::raw('MONTH(created_at) as upload_month'),
            DB::raw('COUNT(*) as upload_count')
        )
        ->groupBy(DB::raw('YEAR(created_at)'), DB::raw('MONTH(created_at)'))
        // ->orderBy('upload_month')
        ->whereNull('deleted_at')
        ->where('bussiness_id',$user_details->bussiness_id )
        ->where( 'created_at' ,'>=', $curent_year.'-01-01' )
        ->get()->toArray();

        $_map = [] ;
        foreach($results as $val){
            $_map[$val['upload_month']] = $val['upload_count'] ;
        }

        $res_ar = [] ;
        for($i=1; $i<=12; $i++ ){
            $res_ar[$i-1] =  isset($_map[$i]) ? $_map[$i] : 0 ;
        }

        return $res_ar ;
    }

}
