<?php

namespace App\Service;

use App\Models\FileDocumentModel;
use App\Models\TemplateFolderAssocModel;
use App\Models\FolderDirectoryModel;
use Faker\Core\Number;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;

class FileDetailService
{
    public static function getDynamicForm(array $data){

        $i_index = json_decode( $data['i_index'] , true);

        if( count($i_index)==0 ){
            return [
                'is_template' => false
            ];
        }

        $template_data = [] ;
        $is_filled = false ;
        if(array_key_exists( 'template_data' , $data  )){

            $template_data = json_decode( $data['template_data'] , true ) ;
            $is_filled = true ;
        }

        $fields_ar = json_decode( $data['fields'] , true );

        $type = [
            1 => 'text',
            // 2 => 'datetime',
            3 => 'date',
            4 => 'time',
            5 => 'dropdown',
            6 => 'check',
            7 => 'radio',
            8 => 'number',
        ];

        $final_ar = [] ;
        foreach($i_index as $val){

            if(!array_key_exists( (int) $fields_ar['s_'.$val] , $type )){
                continue ;
            }

            $value = '' ;

            if( array_key_exists( $fields_ar['l_'.$val] ,$template_data  ) ){
                $value = $template_data[$fields_ar['l_'.$val]] ;
            }

            $ar = [
                'name' => $fields_ar['l_'.$val] ,
                'type' => $type[ $fields_ar['s_'.$val] ] ,
                'label' => $fields_ar['l_'.$val] ,
                'value' => $value,
            ];

            if( $fields_ar['m_'.$val] == 1 ){
                $ar['validators'] =  [
                    'required' => true
                ];
            }

            array_push($final_ar, $ar) ;
        }

        return [
            'status' => true ,
            'form_ar' => $final_ar
        ] ;
    }

    // used by file detail
    public static function checkTemplate($folder_id, $auth_details , $form = false )
    {
        $template_info = TemplateFolderAssocModel::select('fields', 'i_index', 'template.id')
            ->leftJoin('template', 'template.id', '=', 'template_folder_assoc.template_id')
            ->where('template_folder_assoc.folder_id', $folder_id)
            ->where('template.bussiness_id', $auth_details->bussiness_id)
            ->whereNull('deleted_at')->get()->toArray();

        if (count($template_info) == 0) {

            return self::getHeirarchy($folder_id, $auth_details, $form) ;
        }

        $res = FileDetailService::getDynamicForm(['i_index' => $template_info[0]['i_index'], 'fields' => $template_info[0]['fields']]);

        $_val = 'data' ;

        if($form==true){
            $_val = 'form_ar' ;
        }

        return [
            'is_template' => true,
            $_val => $res['form_ar'],
            'template_id' => $template_info[0]['id']
        ];
    }

    // just to check wether any template assigned or not
    public static function checkTemplate2($folder_id, $auth_details , $form = false )
    {

        $template_info = TemplateFolderAssocModel::select('fields', 'i_index', 'template.id')
            ->leftJoin('template', 'template.id', '=', 'template_folder_assoc.template_id')
            ->where('template_folder_assoc.folder_id', $folder_id)
            ->where('template.bussiness_id', $auth_details->bussiness_id)
            ->whereNull('deleted_at')->get()->toArray();

        if (count($template_info) == 0) {

            return self::getHeirarchy($folder_id, $auth_details, $form, true) ;

        }else{

            $fields = json_decode($template_info[0]['fields'], true) ;

            $i_index = json_decode($template_info[0]['i_index'], true) ;

            foreach($i_index as $val ){
                if( isset($fields['m_'.$val]) && $fields['m_'.$val]==true ){
                    return [
                        'is_template' => true,
                    ];
                }
            }

            return [
                'is_template' => false,
            ];
        }
    }

    public static function getHeirarchy($folder_id, $auth_details, $form , $just_check_for_mandat_template_field = false ){

        $f_data = FolderDirectoryModel::select('section_id')->where('id',$folder_id)->first();

        if(!$f_data){
            return [
                'is_template' => false
            ];
        }

        $folder_structure = FolderDirectoryModel::select(
            'id',
            'folder_name',
            'parent_folder_id'
        )->where('section_id', $f_data->section_id)->get()->toArray();

        $data_folder = [];
        foreach ($folder_structure as $val) {
            $data_folder[$val['id']] = $val;
        }

        $id_ar = [];

        $parent_id = $data_folder[$folder_id]['parent_folder_id'];

        while ($parent_id != 0 ) {
            $parent_id = $data_folder[$folder_id]['parent_folder_id'];
            $folder_id = $parent_id;
            $id_ar[] = $parent_id;
        }

        $cnt = count($id_ar) ;

        unset($id_ar[ $cnt-1 ]) ;

        $template_info = TemplateFolderAssocModel::select('fields', 'i_index', 'template.id', 'folder_id')
        ->leftJoin('template', 'template.id', '=', 'template_folder_assoc.template_id')
        ->whereIn('template_folder_assoc.folder_id', $id_ar)
        ->where('template_folder_assoc.is_apply_on_sub_folder', 1)
        ->where('template.bussiness_id', $auth_details->bussiness_id)
        ->whereNull('deleted_at')->get()->toArray();

        if(count( $template_info ) == 0 ){
            return [
                'is_template' => false
            ];
        }

        $get_template_ar = [] ;
        foreach($template_info as $val){
            $get_template_ar[$val['folder_id']]['fields'] = $val['fields'] ;
            $get_template_ar[$val['folder_id']]['id'] = $val['id'] ;
            $get_template_ar[$val['folder_id']]['i_index'] = $val['i_index'] ;
        }

        $res = [] ;
        foreach($id_ar as $val){

            if(array_key_exists($val, $get_template_ar )  ){

                $res = FileDetailService::getDynamicForm([
                        'i_index' =>  $get_template_ar[$val]['i_index'] ,
                        'fields' => $get_template_ar[$val]['fields']
                    ]);
                break ;
            }
        }


        if($just_check_for_mandat_template_field){

            if(isset($res['form_ar'][0]['validators']['required']) ){ // check for has template mand fields or not .
                return [
                    'is_template' => true
                ];
            }else{
                return [
                    'is_template' => false
                ];
            }
        }

        $_val = 'data' ;

        if($form==true){
            $_val = 'form_ar' ;
        }

        return [
            'is_template' => true,
            $_val => $res['form_ar'],
            'template_id' => $template_info[0]['id']
        ];
    }

    // returning : Folder 1 > Folder 2 > Folder 3
    public static function getFilePath($parent_id, $section_id , $section_name){

        if($parent_id == $section_id){
            $data['folder_ids'][0] = 0 ;
            $data['present_dir'][0] = $section_name ;
            return $data ;
        }

        if(!$section_id){

            $data['folder_ids'] = [] ;
            $data['present_dir'] = [] ;
            return $data ;
        }

        $_parent_id = $parent_id ;
        $present_dir = [];
        $root_folder_id = 0;

        $folder_structure = FolderDirectoryModel::select(
            'id',
            'folder_name',
            'parent_folder_id'
        )->where('section_id', $section_id)->get()->toArray();

        $data_folder = [];
        foreach ($folder_structure as $val) {
            $data_folder[$val['id']] = $val;
        }

        $folder_id = $parent_id ;
        $id_ar = [] ;

        while ($parent_id != $root_folder_id) {
            if(isset($data_folder[$folder_id]['parent_folder_id'])){
                $parent_id = $data_folder[$folder_id]['parent_folder_id'];
                $folder_id = $parent_id;
                $id_ar[] = $parent_id;
            }else{
                $parent_id = $root_folder_id ;
            }
        }

        foreach ($id_ar as $val) {
            if ($val != 0) {
                $present_dir[] = $data_folder[$val]['folder_name'];
            }
        }

        $present_dir[count($present_dir)] = $section_name;

        $cnt = count($present_dir) ;
        if ($cnt != count($id_ar)) {
            $id_ar[count($present_dir)] = $root_folder_id;
        }

        $folder_ids = array_reverse( $id_ar ) ;
        $folder_ids[$cnt] = $_parent_id ;

        $present_dir = array_reverse( $present_dir ) ;
        $present_dir[$cnt] = isset($data_folder[$_parent_id]['folder_name']) ? $data_folder[$_parent_id]['folder_name'] : '' ;
        $data['folder_ids'] = $folder_ids ;
        $data['present_dir'] = $present_dir ;

        return $data ;
    }


}


/*
[
  1 : TextField
  2 : DateTime
  3 : Date
  4 : Time
  5 : Dropdown
  6 : CheckBox
  7 : Radio
  8 : Number
]
*/
