<?php

namespace App\Service;

use App\Models\FileDocumentModel;
use App\Models\FolderDirectoryModel;
use App\Models\SectionUserAssocModel;
use App\Models\UserDirPermissionModel;
use Faker\Core\Number;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Models\SectionModel;

class SectionService
{
    public static function getUserPermitFolder(string $section_code ){

    }

    public static function getUserNavigation($user_details ){

        $bsns_id = $user_details->bussiness_id ;

        $section_d = SectionUserAssocModel::select('section_id','section_name','section_code')
        ->leftJoin('section', 'section.id', '=', 'section_user_assoc.section_id')
        ->where('section_user_assoc.user_id',$user_details->id)
        ->where('section.bussiness_id',$bsns_id)->orderBy('section.id','desc')->whereNull('deleted_at')->get()->toArray();

        $user_dir_permission = UserDirPermissionModel::select('section.id','section.section_name','section.section_code')
        ->leftJoin('section', 'section.section_code', '=', 'user_dir_permission.section_code')
        ->where(function ($query) {
            $query->where('view', '1')
                ->orWhere('edit', '1');
        })
        ->where('assigned_to',$user_details->id)
        ->whereNull('user_dir_permission.deleted_at')
        ->whereNull('section.deleted_at')
        ->whereNotNull('section.id')
        ->groupBy('user_dir_permission.section_code')->get()->toArray() ;

        if(count($user_dir_permission)>0){

            $section_id_ar = [];
            foreach($section_d as $val){
                $section_id_ar[] = $val['section_id'] ;
            }

            $cnt = count($section_id_ar) ;

            foreach($user_dir_permission as $val){

                if(!in_array($val['id'], $section_id_ar )){
                    $section_d[$cnt] = $val;
                    $cnt++;
                }
            }
        }

        return $section_d ;
    }

    public static function checkUserFolderAccess($user_details, $section_code, $folder_id, $user_ar, $section_assigned=false){

        $user_dir_permission = UserDirPermissionModel::select('user_dir_permission.folder_id', 'view', 'edit','download','delete')
        ->leftJoin('section', 'section.section_code', '=', 'user_dir_permission.section_code')
        ->where(function ($query) {
            $query->where('view', '1')
                ->orWhere('edit', '1');
        })
        ->where('assigned_to',$user_details->id)
        ->whereNull('user_dir_permission.deleted_at')
        ->whereNull('section.deleted_at')
        ->get()->toArray() ;

        if(count($user_dir_permission) == 0){
            throw new \Exception('Permission Denied');
        }

        $_folder_id_ar = [] ;
        $perm_by_fold_id = [] ;
        foreach($user_dir_permission as $val){
            $_folder_id_ar[] = $val['folder_id'] ;
            $perm_by_fold_id[$val['folder_id']] = $val ;
        }

        $f_det = FolderDirectoryModel::select('id')->whereIn('id',$_folder_id_ar)
        ->whereNull('deleted_at')
        ->where('section_id',$section_code)->get()->toArray() ;

        $folder_structure = FolderDirectoryModel::select( 'id','folder_name',
        'parent_folder_id','added_by','created_at' )->where('section_id',$section_code)->get()->toArray();

        $data_folder = [];
        foreach( $folder_structure as $val ){
            $data_folder[$val['id']] = $val ;
        }

        if($folder_id!=0  ){ // current folder id is not root

            $cur_folder_id = $folder_id ;

            if(in_array( $folder_id, $_folder_id_ar ) ){ // u r in that folder on which permission has been given

                return [
                    'code' => 2,
                    'permission_set' => $perm_by_fold_id[$folder_id]
                ];
            }

            $desired_tree = [];
            $folder_id_ar = [];
            $id_found = false ;
            $id_ar = [];
            foreach($f_det as $val){

                $id_ar = [];
                $folder_id = $val['id'] ;
                $parent_id = $data_folder[$folder_id]['parent_folder_id'] ;

                while( 0 != $folder_id){
                    $parent_id = $data_folder[$folder_id]['parent_folder_id'] ;
                    $folder_id = $parent_id ;
                    $id_ar[] = $parent_id;
                }

                if(in_array( $folder_id , $id_ar )){
                    $desired_tree = $id_ar ;
                    $id_found = true ;
                    break ;
                }
            }

            if($id_found==false && $data_folder[$folder_id]['added_by']!= $user_details->id ){
                throw new \Exception('Permission Denied') ;
            }else if($id_found==false && $data_folder[$folder_id]['added_by'] == $user_details->id ) {
                // user is the owner of the folder in which it tries to go
                return [
                    'code' => 2,
                    'permission_set' => [
                        'edit' => 1,
                        'download' => 1,
                        'delete' => 1
                    ]
                ];
            }

            $present_dir = [];
            foreach($desired_tree as $val){
                if($val!=0){
                    $present_dir[] = $data_folder[$val]['folder_name'] ;
                }
            }

            $present_dir[ count($present_dir) ] = 'Root';

            if(count($present_dir)!= count($id_ar)){
                $id_ar[count($present_dir)] = 1111 ;
            }

            // --- 33 end

            $desired_tree = array_reverse($desired_tree) ;
            $desired_tree[count($desired_tree)] = $f_det[0]['id'] ;

            $_val = array_search($cur_folder_id, $desired_tree) ;

            if($_val==0 || $_val){
                $f_id = $desired_tree[$_val+1] ;
            }

            $new_desired_tree = [] ;
            $present_dir = [] ;
            foreach($desired_tree as $val){
                $new_desired_tree[] = $val ;

                if($val==0){
                    $present_dir[] = 'Root' ;
                }else{
                    $present_dir[] = $data_folder[$val]['folder_name'] ;
                }

                if($val == $cur_folder_id){
                    break ;
                }
            }

            // -- 44 strt
            $cur_folder_id = $f_id ;
            $i = 0;
            $_resp[$i]['is_folder'] = 1;
            $_resp[$i]['folder_id'] = $cur_folder_id;
            $_resp[$i]['name'] = $data_folder[$cur_folder_id]['folder_name']   ;

            if( in_array($cur_folder_id,$_folder_id_ar ) ){
                $_resp[$i]['edit_perm'] =  $perm_by_fold_id[$cur_folder_id]['edit'] == 1 ? true : false ;
                $_resp[$i]['delete_perm'] = $perm_by_fold_id[$cur_folder_id]['delete'] == 1 ? true : false ;
                $_resp[$i]['download_perm'] = $perm_by_fold_id[$cur_folder_id]['download']  == 1 ? true : false ;
            }

            $_resp[$i]['added_by'] = array_key_exists ( $data_folder[$cur_folder_id]['added_by'] ,$user_ar ) ? $user_ar[ $data_folder[$cur_folder_id]['added_by'] ] : '' ;
            $_resp[$i]['created_at'] = date('M d, Y h:i:s a', strtotime( $data_folder[$cur_folder_id]['created_at'] )) ;
            $i++ ;


           return  [
                'files_dir' => $_resp,
                'present_dir' =>   $present_dir ,// ['Root']  ,
                'folder_ids' => $desired_tree,
                'is_template' => false,
                'template_value' =>  null  ,
                'template_id' =>  null,
                // 'explicit_permission' => true ,
                // 'all_permission_deny' => true ,
            ] ;
        }

        $folder_id_ar = [];

        foreach($f_det as $val){

            $id_ar = [];
            $folder_id = $val['id'] ;
            $parent_id = $data_folder[$folder_id]['parent_folder_id'] ;

            if( in_array( $folder_id , $_folder_id_ar) ){  // folder exist direct on root

                $folder_id_ar[] = $folder_id ;

            }else{

                while( 0 != $folder_id ){
                    $parent_id = $data_folder[$folder_id]['parent_folder_id'] ;
                    $folder_id = $parent_id ;
                    $id_ar[] = $parent_id;
                }

                $folder_id_ar[] = $id_ar[1] ;
            }
        }

        if(count($folder_id_ar)==0 && $section_assigned==false){
            throw new \Exception('Permission denied');
        }

        $_resp = [] ;
        $i = 0;
        foreach($folder_id_ar as $folder_id){

            if( in_array($folder_id,$_folder_id_ar ) ){
                $_resp[$i]['edit_perm'] =  $perm_by_fold_id[$folder_id]['edit'] == 1 ? true : false ;
                $_resp[$i]['delete_perm'] = $perm_by_fold_id[$folder_id]['delete'] == 1 ? true : false ;
                $_resp[$i]['download_perm'] = $perm_by_fold_id[$folder_id]['download']  == 1 ? true : false ;
            }

            $_resp[$i]['is_folder'] = 1;
            $_resp[$i]['folder_id'] = $folder_id;
            $_resp[$i]['is_owner'] = false ;
            $_resp[$i]['name'] = $data_folder[$folder_id]['folder_name']   ;
            $_resp[$i]['added_by'] = array_key_exists ( $data_folder[$folder_id]['added_by'] ,$user_ar ) ? $user_ar[  $data_folder[$folder_id]['added_by'] ] : '' ;
            $_resp[$i]['created_at'] =  date('M d, Y h:i:s a', strtotime( $data_folder[$folder_id]['created_at'] )) ;
            $i++ ;
        }

       return  [
            'files_dir' => $_resp,
            'present_dir' => ['Root']  ,
            'folder_ids' => [0],
            'is_template' => false,
            'template_value' =>  null  ,
            'template_id' =>  null,
            'all_permission_deny' => true ,
        ] ;
    }

    // code for admin only not for the other users
    public static function doAdvSearch($request_ar){

        $adv_search_data = $request_ar['adv_search_data'] ;

        $section_id = $request_ar['section_code'];
        $folder_id = $request_ar['folder_id'];
        $to_search =  isset($adv_search_data['search_for']) ? $adv_search_data['search_for'] : null ;

        $folder_data = FolderDirectoryModel::select( 'id','folder_name','created_at',
        'parent_folder_id', 'added_by')
        ->where('folder_name', 'like', '%' . $to_search . '%')
        ->where('section_id',$section_id)->get()->toArray();

        $file_data = FileDocumentModel::select('name', 'unique_id', 'created_at', 'added_by')
        ->where('section_code', $section_id) ;


        if($to_search){
            $file_data->where(function ($query) use ($to_search ) {
                $query->where('name', 'like', '%' . $to_search . '%')
                    ->orWhere('template_data', 'like', '%' . $to_search . '%');
                $query->orWhere('tags', 'like', '%' . $to_search . '%');
                $query->orWhere('title', 'like', '%' . $to_search . '%');
                $query->orWhere('description', 'like', '%' . $to_search . '%');
            }) ;
        }

        $temp_ar = $request_ar['adv_search_data']['template_data'] ;
        if(count($temp_ar)!=0){
            foreach($temp_ar as $k=>$val){
                if($val){
                    //   $file_data->where('template_data->'.$k , 'LIKE', '%'.$val.'%') ;
                    //   $file_data->whereRaw("LOWER(JSON_UNQUOTE(JSON_EXTRACT(template_data, '$.\"Full Name\"'))) LIKE '%bharat%'") ;
                    $file_data->whereRaw("LOWER(JSON_UNQUOTE(JSON_EXTRACT(template_data, '$.\"{$k}\"'))) LIKE '%{$val}%'") ;
                }
            }

            $file_data->where('template_id', $adv_search_data['adv_template_id']);
        }

        if( isset($adv_search_data['added_by'])  && $adv_search_data['added_by']){
            $file_data->where('added_by', $adv_search_data['added_by']);
        }

        if( isset($adv_search_data['adv_file_format'])  && $adv_search_data['adv_file_format']){
            $file_data->where('extension', 'like' , '%'.$adv_search_data['adv_file_format'].'%');
        }

        if($adv_search_data['adv_st_date']){
            $file_data->where('created_at', '>=' , $adv_search_data['adv_st_date']);
        }

        if($adv_search_data['adv_en_date']){
            $file_data->where('created_at', '<=' , $adv_search_data['adv_en_date']);
        }

        return $file_data->get()
        ->toArray();
    }


    public static function getSectionName($sec_code){

        $_ar = SectionModel::select('id','section_name')->where('section_code', $sec_code)->first() ;
        if($_ar){
           return $_ar->section_name ;
        }
        return 'root' ;
    }
}
