<?php

namespace App\Http\Controllers;

use App\Models\FileDocumentModel;
use App\Models\FolderDirectoryModel;
use App\Models\SectionModel;
use App\Models\SectionUserAssocModel;
use App\Models\TemplateFolderAssocModel;
use App\Models\TemplateModel;
use App\Models\LinkDocumentModel;
use App\Models\BussinessModel;
use Faker\Core\Number;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use App\Models\UserDirPermissionModel;
use App\Models\DocumentVersioningModel;
use App\Service\FileDetailService;
use App\Service\HelperFunctionService;
use App\Service\SectionService;
use App\Service\AuditLogService;
use Illuminate\Support\Facades\Crypt;

class CollectionController extends Controller
{
    public const CODE = 'CC';

    /**
     * Author : Bharat
     * For uploading the documents
     */
    public function uploadDocument(Request $request)
    {
        try {
            $user_details = Auth::user();

            if (!$user_details) {
                throw new \Exception('User is not authenticate');
            }

            if (!$user_details->role_id) {
                throw new \Exception('Permission denied');
            }

            // check for plan expiry start
            $bsns_id = $user_details->bussiness_id ;

            $bsns_exp = BussinessModel::select('plan_expired_on')->where('id', $bsns_id )->whereNull('deleted_at')->first() ;

            if(!$bsns_exp){
                throw new \Exception('No Bussiness id found') ;
            }

            if(!$bsns_exp->plan_expired_on){
                Log::info("No plan expiry date found") ;
            }else{
                $exp_date = $bsns_exp->plan_expired_on ;
                $today_date = date('Y-m-d');

                $ts_exp_date = strtotime($exp_date);
                $ts_today = strtotime($today_date);

                if ($ts_exp_date < $ts_today) {
                    return response(['status' => true]);
                //    throw new \Exception('Plan Expired');
                }
            }
            // check for plan expiry start

            $input = $request->all();

            if ( !isset($input['additional_data']) || !$input['additional_data']) {

                if( $request->file != null ){
                    $n = $request->file->getClientOriginalName() ;
                    throw new \Exception('Unable to upload : '.$n );
                }
                throw new \Exception('Unable to upload');
            }

            $_data = json_decode($input['additional_data'], true);

            $current_dir_id = $_data['current_dir_id'];

            $section_code = $_data['section_code'];

            if (!$section_code) {
                throw new \Exception('section_code not found');
            }

            $current_user_id = $user_details->id;

            $uploadedFile = $request->file('file');

            $mimeType = $uploadedFile->getClientMimeType();
            $extenstion = $uploadedFile->getClientOriginalExtension();
            $sizeInKB = $uploadedFile->getSize() / 1024; // Convert bytes to KB

            if($sizeInKB == 0){
                throw new \Exception('Unable to upload');
            }

            $uniq_id = rand(10000000, 99999999);
            $fileName = time() . '_' . $uniq_id . '.' . $extenstion;
            $filePath = $request->file('file')->storeAs('upload_box', $fileName, 'public');

            if ($sizeInKB > 2097152) {
                throw new \Exception('File Can not be greater than the 2 GB');
            }

            $res = FileDocumentModel::create([
                'name' =>  $request->file->getClientOriginalName(),
                'unique_id' => $uniq_id,
                'mimetype' => $mimeType,
                'extension' => $extenstion,
                'size_1' => $sizeInKB,
                'path' => $filePath,
                'folder_id' => $current_dir_id == 0 ? $section_code : $current_dir_id, # S092 = on root otherwise folder_id
                'added_by' => $current_user_id,
                'template_id' => $_data['is_template'] == 1 ? $_data['template_id'] : null,
                'template_data' => $_data['is_template'] == 1 ? json_encode($_data['formData'])  : null,
                'section_code' => $section_code, # S092
                'bussiness_id' => $user_details->bussiness_id,

                // 'size_2' ,
                // 'stored_at',
                // 'public_url',
                // 'parent_folder',
                // 'tag_id',
                // 'title',
                // 'description',
            ]);

            AuditLogService::addAuditLog([
                'file_uid' =>  $uniq_id,
                'user_id' =>  $current_user_id,
                'action' =>   'New document added  ' . $request->file->getClientOriginalName(),
                'folder_id' =>  $current_dir_id ,
                'bussiness_id' => $user_details->bussiness_id,
                'section_code' =>  $section_code ,
                'log_type' => 13 ,
                'entity_type' => 1,
                'text' => 'file upload' ,
            ]) ;

            return response(['status' => true, 'message' => 'File Uploaded successfully.']);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    /**
     * Author : Bharat
     */
    public function collectionList(string $user_id, $folder_id, $section_code)
    {

        try {

            $user_details = HelperFunctionService::doValidateForUser();

            $req = [
                'user_id' => $user_id,
                'folder_id' => $folder_id,
                'section_code' => $section_code,
            ];

            $validation = Validator::make($req, [
                'user_id' => 'required|numeric',
                'folder_id' => 'required|numeric',
                'section_code' => 'required|string'
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            //---

            $bsns_id = $user_details->bussiness_id;
            $section_assigned = false ;

            //------ user name fetch start
            $user_d = User::select('id', 'name')->where('bussiness_id', $bsns_id)->get()->toArray();

            $user_ar = [];
            foreach ($user_d as $k => $val) {
                $user_ar[$val['id']] = $val['name'];
            }
            //------ user name fetch end
            $to_cont = false;
            $permission_set = [] ;
            $is_common = false;
            $perm_folder_ar = [] ; // if any folder has been given permission explicitly while already section is assigned as well
            $cur_folder_owner = false ;

            if($user_details->role_id==1 || $user_details->role_id==2){
                $cur_folder_owner = false ;
            }

            if ($section_code == 'C00' . $bsns_id) {
                $is_common = true;
            } else if (substr($section_code, 0, 1) == 'S') {

                $count = SectionModel::where('section_code', $section_code)->where('bussiness_id', $bsns_id)->count();

                if ($count == 0) {
                    return response()->json([
                        'status' => false,
                        'access_denied' => true
                    ], 200);
                }

                if ($user_details->role_id != 2) { // not group admin

                    $f_data = FolderDirectoryModel::select('added_by')->where('id',$folder_id)->first();

                    if ( $f_data && $f_data->added_by == $user_details->id ){
                        $cur_folder_owner = true ;
                    }

                    $section_id = intval(str_replace('S', '', $section_code));

                    $count = SectionUserAssocModel::where('section_id', $section_id)->where('user_id', $user_details->id)->count();

                    if ($count == 0) {

                        $_data = SectionService::checkUserFolderAccess($user_details, $section_code, $folder_id, $user_ar);

                        if (array_key_exists('code', $_data)) {
                            if ($_data['code'] == 2) {
                                $permission_set = $_data['permission_set'] ;
                                $to_cont = true;
                            }
                        }

                        if (!$to_cont) {
                            return response()->json([
                                'status' => true,
                                'data' => $_data,
                            ], 200);
                        }
                    }else{

                        $section_assigned = true ;

                        // to check user is the owner of the cur_folder
                        $_cnt = FolderDirectoryModel::select('id')->where('id',$folder_id)
                        ->where('added_by',$user_details->id )
                        ->whereNull('deleted_at')
                        ->where('section_id',$section_code)->count() ;

                        if($_cnt==0){ // need to get the folder on the root

                            $_data_ar = SectionService::checkUserFolderAccess($user_details, $section_code, $folder_id, $user_ar, $section_assigned=true);

                            if (array_key_exists('code', $_data_ar)) {
                                if ($_data_ar['code'] == 2) {
                                    $permission_set = $_data_ar['permission_set'] ;
                                    $to_cont = true;
                                }
                            }

                            if( array_key_exists('files_dir', $_data_ar ) ){
                                $perm_folder_ar = $_data_ar['files_dir'] ;
                            }
                        }
                    }
                }
            } else {
                return response()->json([
                    'status' => false,
                    'access_denied' => true
                ], 200);
            }

            //---

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $section_id = $section_code; // must be fetch from user id ;
            $parent_folder_id = $folder_id;
            $user_id = $user_details->id;

            $is_template_ass = false ;
            if($folder_id!=0){
                $check = FileDetailService::checkTemplate2($folder_id, $user_details) ;

                if($check['is_template']){
                    $is_template_ass = true ;
                }
            }


            if ($user_details->role_id == 2 || $is_common || $to_cont ) {  // group admin

                $folder_data = FolderDirectoryModel::select(
                    'id',
                    'folder_name',
                    'created_at',
                    'parent_folder_id',
                    'added_by'
                )->where('parent_folder_id', $parent_folder_id)->whereNull('deleted_at')->where('section_id', $section_id)->get()->toArray();

                $file_data = FileDocumentModel::select('name', 'unique_id', 'created_at', 'added_by', 'template_data', 'template_id' )->where('folder_id', $folder_id == 0 ? $section_id : $folder_id)->whereNull('deleted_at')->whereNotNull('unique_id')->get()->toArray();
            } else {

                $folder_data = FolderDirectoryModel::select(
                    'id',
                    'folder_name',
                    'created_at',
                    'parent_folder_id',
                    'added_by'
                )->where('added_by', $user_id)->whereNull('deleted_at')->where('parent_folder_id', $parent_folder_id)->where('section_id', $section_id)->get()->toArray();

                $file_data = FileDocumentModel::select('name', 'unique_id', 'created_at', 'added_by')
                ->whereNotNull('unique_id')
                ->whereNull('deleted_at')
                ->where('added_by', $user_id)
                ->where('folder_id', $folder_id == 0 ? $section_id : $folder_id)
                ->get()->toArray();

            }

            $_resp = [];

            $i = 0;
            $fold_id_added = [] ;
            foreach ($perm_folder_ar as $val) {
                $_resp[$i] = $val ;
                $i++;
                $fold_id_added[] = $val['folder_id'] ;
            }

            foreach ($folder_data as $val) {
                if(in_array( $val['id'] ,$fold_id_added)){
                    continue ;
                }
                $_resp[$i]['is_folder'] = 1;
                $_resp[$i]['folder_id'] = $val['id'];
                $_resp[$i]['show_tooltip'] = false; // for template
                $_resp[$i]['name'] = $val['folder_name'];
                $_resp[$i]['is_owner'] = $val['added_by']==$user_details->id ? true : false ;
                $_resp[$i]['added_by'] = array_key_exists($val['added_by'], $user_ar) ? $user_ar[$val['added_by']] : '';
                $_resp[$i]['created_at'] =  date('M d, Y h:i:s a', strtotime($val['created_at']));
                $i++;
            }

            foreach ($file_data as $val) { // files
                $_resp[$i]['is_folder'] = 0;
                $_resp[$i]['name'] = $val['name'];
                $_resp[$i]['show_tooltip'] = $is_template_ass ? ($val['template_id'] ? false : true) : false ;

                $_resp[$i]['is_owner'] = $val['added_by']==$user_details->id ? true : false ;
                $_resp[$i]['added_by'] = array_key_exists($val['added_by'], $user_ar) ? $user_ar[$val['added_by']] : '';
                $_resp[$i]['unique_id'] = $val['unique_id'];
                $_resp[$i]['created_at'] =  date('M d, Y h:i:s a', strtotime($val['created_at']));
                $i++;
            }

            $present_dir = [SectionService::getSectionName($section_code)] ;
            // $present_dir_id = [ 0 ] ;
            $id_ar = [intval($folder_id)];

            if ($folder_id) {

                $present_dir = [];
                $root_folder_id = 0;

                $folder_structure = FolderDirectoryModel::select(
                    'id',
                    'folder_name',
                    'parent_folder_id'
                )->where('section_id', $section_id)->get()->toArray();

                $data_folder = [];
                foreach ($folder_structure as $val) {
                    $data_folder[$val['id']] = $val;
                }

                $parent_id = $data_folder[$folder_id]['parent_folder_id'];

                while ($parent_id != $root_folder_id) {
                    $parent_id = $data_folder[$folder_id]['parent_folder_id'];
                    $folder_id = $parent_id;
                    $id_ar[] = $parent_id;
                }

                foreach ($id_ar as $val) {
                    if ($val != 0) {
                        $present_dir[] = $data_folder[$val]['folder_name'];
                    }
                }

                // $present_dir[count($present_dir)] = 'Root';
                $present_dir[count($present_dir)] = SectionService::getSectionName($section_code) ;

                if (count($present_dir) != count($id_ar)) {
                    $id_ar[count($present_dir)] = $root_folder_id;
                }
            }

            // $template_result = FileDetailService::checkTemplate($folder_id, $user_details);
            $template_result['is_template'] = false ;

            $permission_ar = [
                'add_folder' => $to_cont==true ? ( ($permission_set['edit']==1) ? true : false ) : false ,
                'upload_file' => $to_cont==true ? ( ($permission_set['edit']==1) ? true : false ) : false ,
                'edit' => $to_cont==true ? ( ($permission_set['edit']==1) ? true : false ) : false  ,
                'download' => $to_cont==true ? ( ($permission_set['download']==1) ? true : false ) : false  ,
                'delete' => $to_cont==true ? ( ($permission_set['delete']==1) ? true : false ) : false  ,
            ] ;

            return response()->json([
                'status' => true,
                'data' => [
                    'files_dir' => $_resp,
                    'present_dir' => array_reverse($present_dir),
                    'folder_ids' => array_reverse($id_ar),
                    'is_template' => $template_result['is_template'],
                    'template_value' => $template_result['is_template'] == true ? $template_result['data'] : null,
                    'template_id' => $template_result['is_template'] == true ? $template_result['template_id'] : null,
                    'is_admin' => $user_details->role_id == 2 ? true : false  ,
                    'section_assigned' => $section_assigned,
                    'cur_folder_owner' => $cur_folder_owner,
                    'is_common_section' => $is_common,
                    'permission_ar' => $permission_ar
                ],
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '03',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function searchInCollectionList( Request $request ){

        try{

            $user_details = HelperFunctionService::doValidateForUser() ;

            $req = $request->all() ;

            $validation = Validator::make($req,[
                'folder_id'=>'required|numeric',
                'section_code'=>'required|string',
                'filter'=>'required|array',
                'to_search'=>'nullable|string|max:100|regex:/^[a-zA-Z0-9 -]*$/', // alpha numeric with space and hypen are allowed , // ^[a-zA-Z0-9 -]*$
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validated = $validation->validated();

            $adv_search = $request->adv_search;

            $section_code = $req['section_code'];
            $folder_id = $req['folder_id'];
            $to_search = $req['to_search'];

            //---
            $bsns_id = $user_details->bussiness_id ;
            //------ user name fetch start
            $user_d = User::select('id','name')->where('bussiness_id',$bsns_id)->get()->toArray();

            $user_ar = [] ;
            foreach($user_d as $k=>$val){
                $user_ar[$val['id']] = $val['name'] ;
            }
            //------ user name fetch end

            $is_common = false ;
            if($section_code == 'C00'.$bsns_id){
                $is_common = true ;
            }else if( substr($section_code, 0, 1) == 'S' ){

                $count = SectionModel::where('section_code',$section_code)->where('bussiness_id',$bsns_id)->count() ;

                if($count==0){
                    throw new \Exception('Permission denied');
                }

                if($user_details->role_id!=2){

                    $section_id = intval(str_replace('S', '',$section_code) ) ;

                    $count = SectionUserAssocModel::where('section_id',$section_id)->where('user_id',$user_details->id)->count() ;

                    if($count==0){
                        throw new \Exception('Permission Denied');
                    }
                }

            }else{
                throw new \Exception('Permission denied');
            }

            //---
            $file_data = [] ;
            $section_id = $section_code ; // must be fetch from user id ;
            $parent_folder_id = $folder_id ;
            $user_id = $user_details->id ;
            $folder_data = [] ;

            $filter = $validated['filter'] ;

            if($adv_search){
                $file_data = SectionService::doAdvSearch($req) ;
            }else{

                if($user_details->role_id==2 || $is_common ){  // group admin

                    $folder_data = [] ;

                    if($filter['fol']){

                        $folder_data = FolderDirectoryModel::select( 'id','folder_name','created_at',
                        'parent_folder_id', 'added_by')
                        ->where('folder_name', 'like', '%' . $to_search . '%')
                        ->where('section_id',$section_id)->get()->toArray();
                    }

                    if($filter['doc']){

                        $file_data = FileDocumentModel::select('name', 'unique_id', 'created_at', 'added_by')
                        ->whereNull('deleted_at')
                        ->where(function ($query) use ($to_search , $filter ) {
                            $query->where('name', 'like', '%' . $to_search . '%')
                                ->orWhere('template_data', 'like', '%' . $to_search . '%');
                            if ($filter['tag']) {
                                $query->orWhere('tags', 'like', '%' . $to_search . '%');
                            }
                            if ($filter['title']) {
                                $query->orWhere('title', 'like', '%' . $to_search . '%');
                            }
                            if ($filter['desc']) {
                                $query->orWhere('description', 'like', '%' . $to_search . '%');
                            }
                        })
                        ->where('section_code', $section_id)
                        // ->where('folder_id', $folder_id == 0 ? $section_id : $folder_id)
                        ->get()
                        ->toArray();
                    }

                }else{

                    if($filter['fol']){

                        $folder_data = FolderDirectoryModel::select( 'id','folder_name','created_at',
                        'parent_folder_id', 'added_by')
                        ->where('folder_name', 'like', $to_search)
                        ->where('added_by',$user_id)->where('section_id',$section_id)->get()->toArray();
                    }

                    if($filter['doc']){

                        $file_data = FileDocumentModel::select('name', 'unique_id', 'created_at', 'added_by')
                        ->whereNull('deleted_at')
                        ->where(function ($query) use ($to_search , $filter ) {
                            $query->where('name', 'like', '%' . $to_search . '%')
                                ->orWhere('template_data', 'like', '%' . $to_search . '%') ;

                                if ($filter['tag']) {
                                    $query->orWhere('tags', 'like', '%' . $to_search . '%');
                                }

                                if ($filter['title']) {
                                    $query->orWhere('title', 'like', '%' . $to_search . '%');
                                }
                                if ($filter['desc']) {
                                    $query->orWhere('description', 'like', '%' . $to_search . '%');
                                }

                        })
                        ->where('added_by',$user_id)
                        ->where('folder_id',$folder_id==0 ? $section_id : $folder_id )
                        ->where('section_code', $section_id)
                        ->get()
                        ->toArray();
                    }

                    // $file_data = FileDocumentModel::select('name','unique_id','created_at','added_by')
                    // ->where(function($query)  use($to_search)  {
                    //     $query->where('name', 'like', '%' . $to_search . '%')
                    //           ->orWhere('template_data', 'like', '%' . $to_search . '%');
                    // })
                    // ->where('added_by',$user_id)->where('folder_id',$folder_id==0 ? $section_id : $folder_id )->get()->toArray();
                }
            }

            $_resp = [] ;

            $i = 0 ;

            foreach($folder_data as $val){
                $_resp[$i]['is_folder'] = 1;
                $_resp[$i]['folder_id'] = $val['id'];
                $_resp[$i]['name'] = $val['folder_name'];
                $_resp[$i]['added_by'] = $user_ar[$val['added_by']];
                $_resp[$i]['created_at'] =  date('M d, Y h:i:s a', strtotime( $val['created_at'] )) ;
                $i++ ;
            }

            foreach($file_data as $val){ // files
                $_resp[$i]['is_folder'] = 0;
                $_resp[$i]['name'] = $val['name'];
                $_resp[$i]['added_by'] = $user_ar[$val['added_by']];
                $_resp[$i]['unique_id'] = $val['unique_id'];
                $_resp[$i]['created_at'] =  date('M d, Y h:i:s a', strtotime( $val['created_at'] )) ;
                $i++ ;
            }

            $present_dir = ['Root'] ;
            // $present_dir_id = [ 0 ] ;
            $id_ar = [ intval($folder_id)  ];

            if($folder_id){

                $present_dir = [];
                $root_folder_id = 0 ;

                $folder_structure = FolderDirectoryModel::select( 'id','folder_name',
                'parent_folder_id')->where('section_id',$section_id)->get()->toArray();

                $data_folder = [];
                foreach( $folder_structure as $val ){
                    $data_folder[$val['id']] = $val ;
                }

                $present_dir[ 0 ] = 'Root';
            }

            return response()->json([
                'status'=> true,
                'data'=> [
                    'files_dir' => $_resp,
                    'present_dir' => [] ,
                    'folder_ids' => []
                ],
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'03',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=> false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function checkIfPlanExpired(Request $request){

        try{

            $user_details = Auth::user();

            if (!$user_details) {
                throw new \Exception('User is not authenticate');
            }

            if (!$user_details->role_id) {
                throw new \Exception('Permission denied');
            }

            // check for plan expiry start
            $bsns_id = $user_details->bussiness_id ;

            $bsns_exp = BussinessModel::select('plan_expired_on')->where('id', $bsns_id )->whereNull('deleted_at')->first() ;

            if(!$bsns_exp){
                throw new \Exception('No Bussiness id found') ;
            }

            if(!$bsns_exp->plan_expired_on){
                // Log::info("CC02 :: No plan expiry date found") ;
                throw new \Exception('No plan expiry date found') ;
            }else{
                $exp_date = $bsns_exp->plan_expired_on ;
                $today_date = date('Y-m-d');

                $ts_exp_date = strtotime($exp_date);
                $ts_today = strtotime($today_date);

                if ($ts_exp_date < $ts_today) {
                    return response(['status' => false, 'admin_msg' => 'Your Plan has Expired, Buy the Plan.', 'user_msg' => 'The plan has expired. Please consult with the administrator.'  ]);
                }
            }
            // check for plan expiry start

            return response(['status' => true]);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'04',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=> false,
                'message'=> $e->getMessage()
            ],200);
       }

    }

}
