<?php

namespace App\Http\Controllers;

use App\Models\FolderDirectoryModel;
use App\Models\User;
use App\Models\UserDirPermissionModel;
use Illuminate\Support\Facades\Validator;
use App\Service\HelperFunctionService;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Models\AuditLogModel;
use App\Models\SectionModel;
use App\Service\DashboardService;



class DashboardController extends Controller
{

    public const CODE = 'DC' ;

    public function getDashboardData(Request $request){

        try{
            
            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;

            $_res = DashboardService::getAvailStorage($user_details) ;

            $space_consumption = $_res['data'] ;

            $uploaded_docs = DashboardService::totalUploadedDocs($user_details) ;

            $total_uploads = $uploaded_docs['total_uploads'] ;

            $total_users_by_plan = $_res['total_users'];

            $created_users = $uploaded_docs['total_users'];

            $_res_section_space_consumption = DashboardService::sectionSpaceConsumption($user_details) ;

            $_res_doc_format = DashboardService::docFormat($user_details) ;

            $monthWiseUploads = DashboardService::monthWiseUploads($user_details) ;

            // $plan_remaining_days = DashboardService::planExpiryDays($user_details) ;

            $c_data = [
                'user_data' => [
                    'total' =>  $_res['total_users'],
                    'used' =>  $uploaded_docs['total_users']
                ],
                'total_uploads' => $total_uploads,

                'space' => $space_consumption ,

                'documentUploaded' => [
                    'monthsChart' =>  ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul','Aug', 'Sep', 'Oct', 'Nov', 'Dec'],
                    'data' => $monthWiseUploads,
                ],

                'userAdded' => [
                    'monthsChart' =>  ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul'],
                    'data' => [10, 20, 12, 39, 27],
                ],

                'monthUploads' => [
                    'data' => [
                        [
                            'name' => 'Image (100)' ,
                            'y' => 100
                        ],
                        [
                            'name' => 'Pdf (50)' ,
                            'y' => 50
                        ],
                        [
                            'name' => 'Excel (100)' ,
                            'y' => 100
                        ],
                        [
                            'name' => 'Other (20)' ,
                            'y' => 20
                        ],
                    ] 
                ],
                'availableSpace' => [
                    'data' => [
                        [
                            'name' => "Free Space ( {$space_consumption['free']} )" ,
                            'y' => $space_consumption['free']
                        ],
                        [
                            'name' => "Used Space ( {$space_consumption['used']} )" ,
                            'y' => $space_consumption['used']
                        ],
                    ]
                ],
                'section_vise_space' => [
                    'data' => $_res_section_space_consumption['space']
                ],
                'this_month_uploads' => $_res_doc_format,
                'rem_days' => $_res['rem_days']
            ] ;

            return response()->json([
                'status' => true,
                'data' => $c_data
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $err
            ],200);
       }
    }
}
