<?php

namespace App\Http\Controllers;

use App\Models\FileDocumentModel;
use App\Models\FolderDirectoryModel;
use App\Models\SectionModel;
use App\Models\SectionUserAssocModel;
use App\Models\TemplateFolderAssocModel;
use App\Models\TemplateModel;
use App\Models\LinkDocumentModel;
use Faker\Core\Number;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use App\Models\UserDirPermissionModel;
use App\Models\DocumentVersioningModel;
use App\Service\FileDetailService;
use App\Service\HelperFunctionService;
use App\Service\SectionService;
use App\Service\AuditLogService;
use App\Service\MultiService;
use App\Models\DocumentCommentModel;

class FileController extends Controller
{
    public const CODE = 'FC';

    public function fileDetail(Request $request)
    {

        try {

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all();

            $_details = FileDocumentModel::select(
                'id',
                'extension',
                'template_data',
                'template_id',
                'name',
                'created_at',
                'updated_at',
                'added_by',
                'tags',
                'title',
                'section_code',
                'is_locked',
                'locked_by',
                'folder_id',
                'updated_by',
                'size_1',
                'description',
                'retention_period',
                'retention_date',
                'retention_type',
            )->where('unique_id', $req['fid'] )->whereNull('deleted_at')->first();

            if (!$_details) {
                throw new \Exception('File not found');
            }

            $template_data = [
                'is_template' => false
            ];

            if ($_details->template_id) {

                $templ_data = TemplateModel::select('fields', 'i_index')->where('id', $_details->template_id)->first();

                if ($templ_data) {

                    $res = FileDetailService::getDynamicForm(['i_index' => $templ_data->i_index, 'fields' => $templ_data->fields, 'template_data' => $_details->template_data]);

                    $template_data = [
                        'is_template' => true,
                        'form_ar' => $res['form_ar'],
                        'template_id' => $_details->template_id
                    ];
                }
            }else{

                $template_data = FileDetailService::checkTemplate( $_details->folder_id ,$user_details , true ) ;
            }

            //--- user st
            $added_by = $_details->added_by ;
            $updated_by = $_details->updated_by ;

            $user_d = User::select('name','id')->whereIn('id',[$added_by, $updated_by] )->get() ;

            $user_ar = [];
            foreach($user_d as $val){
                $user_ar[$val['id']] = $val['name'] ;
            }
            //--- user en

            if(  $_details->section_code == $_details->folder_id){
                // get section name
            }else{
                $f_data = FolderDirectoryModel::select('folder_name')->where('id',$_details->folder_id)->first();
            }

            $is_owner = $_details->added_by == $user_details->id ? true : false ;

            $is_admin =  $user_details->role_id == 2 ? true : false  ;

            $det_ar = $_details->toArray();
            $det_ar['created_at'] = date('d M Y h:i:s a', strtotime($_details->created_at));
            $det_ar['updated_at'] = date('d M Y h:i:s a', strtotime($_details->updated_at));
            $det_ar['size_1'] = round($_details->size_1, 2) .' kB' ;
            $det_ar['added_by'] = $user_ar[$added_by] ?? null  ;
            $det_ar['updated_by'] = $updated_by ? $user_ar[$updated_by] : null ;
            $det_ar['is_owner'] = $is_owner ;
            $det_ar['is_admin'] = $user_details->role_id == 2 ? true : false  ;
            $det_ar['folder_name'] = '' ; // $f_data ? $f_data->folder_name : null;

            $det_ar['retention_type'] = $_details->retention_type ;
            $det_ar['retention_period'] = $_details->retention_period ;
            $det_ar['retention_date'] = $_details->retention_date ;

            if($det_ar['retention_type']==2){
                $ar_exp = explode(',', $_details->retention_period ) ;
                $det_ar['r_days'] = $ar_exp[0] ;
                $det_ar['r_month'] = $ar_exp[1] ;
                $det_ar['r_yr'] = $ar_exp[2] ;
            }

            $lock_date_time = $det_ar['is_locked'] ;
            $det_ar['is_locked'] = $det_ar['is_locked'] ? true : false ;
            $det_ar['locked_by_me'] = ($det_ar['locked_by']==$user_details->id) ? true : false ;

            $permission_set = [
                'edit' => ( $is_owner || $is_admin )  ? true : false ,
                'download' => ( $is_owner || $is_admin ) ? true : false ,
                'delete' => ( $is_owner || $is_admin ) ? true : false ,
                'lock' => ( $is_owner || $is_admin ) ? true : false ,
                'upload_new_version' => ( $is_owner || $is_admin ) ? true : false ,
                'link_document' => ( $is_owner || $is_admin ) ? true : false ,
            ];

            //-- check for unlock file strt
            if($det_ar['is_locked']){

                if( time() >= strtotime($lock_date_time) ){

                    $v_data = FileDocumentModel::where('unique_id', $req['fid'] )
                    ->where('bussiness_id' , $user_details->bussiness_id)
                    ->update([
                        'is_locked' => null,
                        'locked_by' => null
                    ]);
                    $det_ar['is_locked'] = false ;
                    $det_ar['locked_by_me'] = false ;
                }
            }
            //-- check for unlock file end

            //--- path of the file strt

            $section_name = 'Common' ;
            if($_details->section_code ){
                $s_data = SectionModel::select('section_code','section_name','id')->where('section_code', $_details->section_code)->first();
                if($s_data){
                    $section_name = $s_data->section_name ;
                }
            }

            $_heir = FileDetailService::getFilePath( $_details->folder_id, $_details->section_code, $section_name ?? 'Root' ) ;
            //--- path of the file end

            if(!$is_owner && !$is_admin){

                $cnt = count($_heir['folder_ids']) ;
                if($cnt==1){
                    throw new \Exception('Access Denied') ;
                }

                $folder_ids = array_reverse($_heir['folder_ids']) ;
                unset($folder_ids[$cnt-1]) ;

                $chk_user_perm = UserDirPermissionModel::select( 'view',
                'edit',
                'download',
                'delete')
                ->where(function ($query) {
                    $query->where('view', '1')
                        ->orWhere('edit', '1');
                })
                ->whereIn('folder_id',$folder_ids)
                ->where('assigned_to',$user_details->id)
                ->whereNull('user_dir_permission.deleted_at')
                ->first() ;

                if(!$chk_user_perm){

                    return response()->json([
                            'status' => true,
                            'access_denied' => true,
                        ], 200);
                }

                $user_perm = $chk_user_perm->toArray() ;

                $permission_set = [
                    'edit' => $user_perm['edit'] ? true : false ,
                    'download' => $user_perm['edit'] ? true : false ,
                    'delete' => $user_perm['delete'] ? true : false ,
                    'lock' => $user_perm['edit'] ? true : false ,
                    'upload_new_version' => $user_perm['edit'] ? true : false ,
                    'link_document' => $user_perm['edit'] ? true : false ,
                ];

            }

            if($req['first_view']){
                AuditLogService::addAuditLog([
                    'file_uid' => $req['fid'] ,
                    'user_id' =>  $user_details->id,
                    'bussiness_id' => $user_details->bussiness_id,
                    'folder_id' => $_details->folder_id == $_details->section_code  ? 0 : $_details->folder_id ,
                    'action' => "<b>File View : </b>".$_details->name ,
                    'section_code' =>  $_details->section_code ,
                    'entity_type' => 1,
                    'log_type' => 19,
                    'text' => 'file view'
                ]) ;
            }

            return response()->json([
                'status' => true,
                'data' => [
                    'file_details' => $det_ar,
                    'template_data' => $template_data,
                    'permission_set' => $permission_set,
                    'file_path' => $_heir,
                    'section_code' => $_details->section_code,
                    'section_name' => $section_name,
                ],
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '07',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function updateFileDetail(Request $request)
    {

        try {

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all();

            $validation = Validator::make($req, [
                'file_detail' => "array|required",
                'retention_set' => "array|required",  // 1: date , 2: period, 3: no ret
                'template_detail' => "array|nullable",
                'template_id' => "required|numeric",
                'is_template' => "required",
                'file_uid' => "required",
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            if ($validated['file_uid'] == 0) {
                throw new \Exception('File id not valid');
            }

            $retention_type = $validated['retention_set']['retention_type'] ;
            $ret_date = '' ;
            $retention_period = '' ;

            if($validated['retention_set']['retention_type']!=3){

                if($validated['retention_set']['retention_type']==1){
                    if($validated['retention_set']['retention_date'] == ''){
                        throw new \Exception('Retention date is invalid.') ;
                    }
                    $ret_date = $validated['retention_set']['retention_date'] ;
                }

                if($validated['retention_set']['retention_type']==2){
                    $retention_period = $validated['retention_set']['r_days'].','.$validated['retention_set']['r_month'].','.$validated['retention_set']['r_yr'] ;

                    $ret_date = date('Y-m-d') ;
                    if(isset($validated['retention_set']['r_days'] )){
                        $ret_date = date('Y-m-d', strtotime('+ '.$validated['retention_set']['r_days'].' days', strtotime($ret_date))) ;
                    }
                    if(isset($validated['retention_set']['r_month'] )){
                        $ret_date = date('Y-m-d', strtotime('+ '.$validated['retention_set']['r_month'].' months', strtotime($ret_date))) ;
                    }
                    if(isset($validated['retention_set']['r_yr'] )){
                        $ret_date = date('Y-m-d', strtotime('+ '.$validated['retention_set']['r_yr'].' years', strtotime($ret_date))) ;
                    }
                }
            }

            $upd_ar = [
                'updated_by' => $user_details->id,
                'template_id' => $validated['is_template'] == true ? $validated['template_id'] : null ,
                'template_data' => $validated['is_template'] == true ? json_encode($validated['template_detail'])  : null,
                'title' => $validated['file_detail']['title'],
                'description' => $validated['file_detail']['desc'],
                'tags' => $validated['file_detail']['tags'],
                'retention_period' => $retention_period,
                'retention_date' => $ret_date,
                'retention_type' => $retention_type,
            ] ;

            FileDocumentModel::where('unique_id', $validated['file_uid'])->update($upd_ar);

            $file_data = FileDocumentModel::select('section_code','name', 'extension','folder_id')->where('unique_id', $validated['file_uid'])->first() ;

            AuditLogService::addAuditLog([
                'file_uid' => $validated['file_uid'],
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $file_data->folder_id == $file_data->section_code  ? 0 : $file_data->folder_id ,
                'action' =>  $file_data->name . "<b> updated </b>" ,
                'section_code' =>  $file_data->section_code ,
                'entity_type' => 1,
                'log_type' => 5 ,
                'text' => 'file update' ,
            ]) ;

            return response()->json([
                'status' => true,
                'message' => 'File Detail Updated successfully.'
            ], 200);
        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '08',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function delFile($file_id)
    {

        try {

            // $user_details = HelperFunctionService::doValidateForGroupAdmin();
            $user_details = HelperFunctionService::doValidateForUser();

            $req = [
                'file_id' => $file_id
            ];

            $validation = Validator::make($req, [
                'file_id' => 'required|numeric' # uniq id
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            $bsns_id = $user_details->bussiness_id;

            //---

            $fil_det = FileDocumentModel::select( 'name', 'extension', 'section_code', 'folder_id' , 'is_locked' ,'locked_by' )->where('unique_id', $validated['file_id'])->where('bussiness_id', $bsns_id)->first() ;

            if(!$fil_det){
                throw new \Exception('Invalid File');
            }

            if($fil_det->is_locked && $fil_det->locked_by!=$user_details->id ){

                if(  (time() <= strtotime($fil_det->is_locked)) ){
                    throw new \Exception('File is locked') ;
                }
            }

            FileDocumentModel::where('unique_id', $validated['file_id'])->where('bussiness_id', $bsns_id)->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            // audit log start

            $file_name = $fil_det->name;

            $action_msg = "$file_name <b>deleted</b>";

            AuditLogService::addAuditLog([
                'file_uid' => $validated['file_id'],
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $fil_det->folder_id ?? 0  ,
                'action' => $action_msg,
                'section_code' =>  $fil_det->section_code ,
                'entity_type' => 1,
                'log_type' => 14 ,
                'text' => 'file del' ,
            ]) ;

            // audit log end

            return response()->json([
                'status' => true,
                'message' => 'File deleted successfully.'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '08',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function downloadFile($file_id, $is_uid = 1 ){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $fld = 'unique_id' ;
            if($is_uid==0){
                $fld = 'id' ;
            }

            $file_data = FileDocumentModel::select('name','unique_id', 'path','folder_id','section_code')->where($fld, $file_id )->whereNull('deleted_at')
            ->where('bussiness_id',$user_details->bussiness_id)
            ->first();

            if(!$file_data){
                throw new \Exception('Some error occured') ;
            }

            // https://filefort.in/filefort-backend/public/storage/upload_box/1703829777_54519670.jpeg # in working for prod
            // http://localhost:8000/storage/upload_box/1697611222_43663113.mp3 # in working
            if(!env("DOWNLOAD_URL") ){
                throw new \Exception('download url not configured') ;
            }

            $url = env("DOWNLOAD_URL") . $file_data->path ;
            // $url =  env('APP_URL') .':'.$_SERVER['SERVER_PORT'].'/storage/'.$file_data->path ; // working in local

            AuditLogService::addAuditLog([
                'file_uid' => $file_data->unique_id,
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $file_data->folder_id == $file_data->section_code  ? 0 : $file_data->folder_id ,
                'action' =>  $file_data->name . " <b>downloaded</b> " ,
                'section_code' =>  $file_data->section_code ,
                'entity_type' => 1,
                'log_type' => 4 ,
                'text' => 'file download' ,
            ]) ;

            return response()->json([
                'status' => true,
                'data' => [
                    'file_name' => $file_data->name ,
                    'url' => $url
                ]
            ], 200);

        }catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '10',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function saveLinkDocs(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all();

            $validation = Validator::make($req, [
                'cur_file_id' => "string|required",
                'm_document_number' => "numeric|required",
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            if ($validated['cur_file_id'] == 0) {
                throw new \Exception('File id not valid');
            }

            if($validated['cur_file_id'] == $validated['m_document_number'] ){
                throw new \Exception('Linked document id can not be same as the current file id.');
            }

            $file_data = FileDocumentModel::select('name', 'id', 'unique_id', 'folder_id', 'section_code')
            ->where('bussiness_id', $user_details->bussiness_id )
            ->where(function ($query) use ($validated) {
                $query->where('unique_id', $validated['m_document_number'])
                    ->orWhere('unique_id', $validated['cur_file_id']);
            })
            ->whereNull('deleted_at')->get()->toArray() ;

            $_ar = [];
            $_ar_2 = [];
            $m_document_number_exist = false ;
            foreach($file_data as $val){
                if($val['unique_id'] == $validated['cur_file_id']){
                    $_ar = (object)$val ;
                }
                if($val['unique_id'] == $validated['m_document_number']){
                    $_ar_2 = (object)$val ;
                    $m_document_number_exist = true ;
                }
            }

            if(!$m_document_number_exist){
                throw new \Exception('File id not valid');
            }

            LinkDocumentModel::create([
                'added_by' => $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'attached_document_id' =>  $validated['m_document_number'],
                'file_id' => $validated['cur_file_id'],
            ]);

            AuditLogService::addAuditLog([
                'file_uid' => $validated['cur_file_id'],
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $_ar->folder_id == $_ar->section_code  ? 0 : $_ar->folder_id ,
                'action' =>  $_ar->name . " <b>linked with</b> ". $_ar_2->name ,
                'section_code' =>  $_ar->section_code ,
                'entity_type' => 1,
                'log_type' => 18,
                'text' => 'file link document'
            ]) ;

            return response()->json([
                'status' => true,
                'message' => 'Document Linked successfully.'
            ], 200);


        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '12',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }


    public function getLinkedDocs($file_uid){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $req['cur_file_id'] = $file_uid;

            $validation = Validator::make($req, [
                'cur_file_id' => "numeric|required"
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            if ($validated['cur_file_id'] == 0) {
                throw new \Exception('File id not valid');
            }

            $linked_doc_info = LinkDocumentModel::select( 'link_document.id', 'link_document.file_id','link_document.attached_document_id')
            ->where(function ($query) use ($file_uid) {
                $query->where('link_document.file_id', $file_uid )
                    ->orWhere('link_document.attached_document_id', $file_uid );
            })
            ->where('link_document.bussiness_id', $user_details->bussiness_id)
            ->whereNull('link_document.deleted_at')
            ->get()->toArray();

            $i = 0 ;
            $ar = [] ;
            $ar_2 = [] ;
            foreach($linked_doc_info as $val){

                if($val['attached_document_id']!=$file_uid){
                    $ar[$val['attached_document_id']] = $val['id'] ;
                    $ar_2[] = $val['attached_document_id'] ;
                }else{
                    $ar[$val['file_id']] = $val['id'] ;
                    $ar_2[] = $val['file_id'] ;
                }
                $i++ ;
            }

            $doc = FileDocumentModel::select('name','unique_id')
            ->whereIn('unique_id', $ar_2 )->get()->toArray();

            $_data = [] ;
            $i = 0 ;
            foreach($doc as $val){
                $_data[$i]['unique_id'] = $val['unique_id'] ;
                $_data[$i]['name'] = $val['name'] ;
                $_data[$i]['id'] = $ar[$val['unique_id']] ;
                $i++ ;
            }

            return response()->json([
                'status' => true,
                'data' => $_data
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '13',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function delLinkedDocs($link_id){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            if(!$link_id){
                throw new \Exception('Invalid Request.') ;
            }

            $_d = LinkDocumentModel::select('file_id')->where('id',$link_id)->first() ;


            $res = LinkDocumentModel::where('id',$link_id)
            ->where('bussiness_id', $user_details->bussiness_id)
            ->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]) ;

            // -- audit log start

            $_f_data = FileDocumentModel::select('section_code','name','unique_id' ,'folder_id')->where('unique_id', $_d->file_id)->first();

            if(!$_f_data){
                throw new \Exception('File not Found.') ;
            }

            AuditLogService::addAuditLog([
                'file_uid' => $_f_data->unique_id ,
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $_f_data->folder_id == $_f_data->section_code  ? 0 : $_f_data->folder_id ,
                'action' =>  " <b>Document Unlinked : </b>".$_f_data->name  ,
                'section_code' =>  $_f_data->section_code ,
                'entity_type' => 1,
                'log_type' => 20,
                'text' => 'file del linked document '
            ]) ;
            // -- audit log end

            return response()->json([
                'status' => true,
                'message' => 'Document Unlinked successfully.'
            ]);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '13',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }


    public function uploadVersion(Request $request)
    {
        try {

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all();

            $validation = Validator::make($req, [
                'cur_file_id' => "string|required", // file_uid
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => $validation->errors()
                ], 200);
            }

            $validated = $validation->validated();

            if ($validated['cur_file_id'] == 0) {
                throw new \Exception('File id not valid');
            }

            // -- 11
            if (!$request->file()) {
                throw new \Exception('File is mandatory');
            }

            $uploadedFile = $request->file('file');

            $mimeType = $uploadedFile->getClientMimeType();
            $extenstion = $uploadedFile->getClientOriginalExtension();
            $sizeInKB = $uploadedFile->getSize() / 1024; // Convert bytes to KB
            $uniq_id = rand(10000000, 99999999);
            $fileName = time() . '_' . $uniq_id . '.' . $extenstion;
            $filePath = $request->file('file')->storeAs('upload_box', $fileName, 'public');

            if ($sizeInKB > 2097152) {
                throw new \Exception('File Can not be greater than the 2 GB');
            }
            if($sizeInKB==0){
                throw new \Exception('File has no size.');
            }

            $file_data =  FileDocumentModel::where('unique_id', $validated['cur_file_id'])->first() ;

            if(!$file_data){
                throw new \Exception('Something Went Wrong') ;
            }

            $new_file_id = FileDocumentModel::create([
                'name' =>  $request->file->getClientOriginalName(),
                'unique_id' => $file_data->unique_id,
                'mimetype' => $mimeType,
                'extension' => $extenstion,
                'size_1' =>  round($sizeInKB, 3) ,
                'path' => $filePath,
                'folder_id' =>  $file_data->folder_id ,
                'added_by' => $user_details->id ,
                'template_id' => $file_data->template_id  ,
                'title' => $file_data->title ,
                'description' => $file_data->description ,
                'tags' => $file_data->tags ,
                'template_data' => $file_data->template_data ,
                'section_code' => $file_data->section_code ,
                'bussiness_id' => $user_details->bussiness_id,
            ])->id;

            FileDocumentModel::where('id', $file_data->id )->update([
                'unique_id' => null
            ]);

            $version_no = 1 ;

            $_count = DocumentVersioningModel::where('file_uid', $validated['cur_file_id'] )->count() ;

            if($_count){
                $version_no = $_count + 1;
            }

            DocumentVersioningModel::create([
                'file_id' => $file_data->id, // main-file , file-uid
                'bussiness_id' => $user_details->bussiness_id,
                'added_by' => $user_details->id ,
                'version_number' => $version_no ,
                'file_uid' => $validated['cur_file_id']
            ]);

            // -- 11 end

            // --- audit log start

            AuditLogService::addAuditLog([
                'file_uid' => $file_data->unique_id,
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $file_data->folder_id == $file_data->section_code  ? 0 : $file_data->folder_id ,
                'section_code' =>  $file_data->section_code ,
                'action' => "<b>New file version upload</b> as : ". $request->file->getClientOriginalName(),
                'entity_type' => 1,
                'log_type' => 13,
                'text' => 'file version upload'
            ]) ;

            // --- audit log end

            return response()->json([
                'status' => true,
                'message' => 'Version Uploaded Successfully.'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '18',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $err
            ], 200);
        }
    }

    public function getOldVersion($file_uid){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $doc_version_info = DocumentVersioningModel::select( 'documents.id as doc_id', 'document_versioning.id as version_id',
                'document_versioning.added_by',
                'document_versioning.version_number',
                // 'document_versioning.created_at',
                'documents.mimetype',
                'documents.name',
                'documents.size_1'
            )
            ->leftJoin('documents', 'document_versioning.file_id', '=', 'documents.id')
            ->where('document_versioning.file_uid', $file_uid )
            ->where('documents.bussiness_id', $user_details->bussiness_id)
            ->orderBy('version_number','desc')
            ->whereNull('document_versioning.deleted_at')
            ->whereNull('documents.deleted_at')->get()->toArray();

            return response()->json([
                'status' => true,
                'data' => $doc_version_info
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '19',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function delOldVersion($v_id){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $d = DocumentVersioningModel::select('file_id')->where('id', $v_id)
            ->where('bussiness_id' , $user_details->bussiness_id)
            ->first() ;

            $file_data = FileDocumentModel::select('path')->where('id', $d->file_id)
            ->where('bussiness_id' , $user_details->bussiness_id)
            ->first() ;

            if(!$d || !$file_data){
                throw new \Exception('Version file not found.') ;
            }

            $v_data = DocumentVersioningModel::where('id', $v_id)
            ->where('bussiness_id' , $user_details->bussiness_id)
            ->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            FileDocumentModel::where('id', $d->file_id)
            ->where('bussiness_id' , $user_details->bussiness_id)
            ->delete();

            if(!env('FULL_PATH_DOCUMENT_UPLOAD')){
                Log::info("ER0013 :: FULL_PATH_DOCUMENT_UPLOAD not found.");
            }

            $path = $file_data->path ;

            $f_path = env('FULL_PATH_DOCUMENT_UPLOAD').$path ;

            unlink($f_path) ;

            return response()->json([
                'status' => true,
                'message' => 'Version deleted successfully.'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '19',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function lockFile($file_uid){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            if(!$file_uid || $file_uid == 0){
                throw new \Exception('Invalid file');
            }

            $v_data = FileDocumentModel::where('unique_id', $file_uid)
            ->where('bussiness_id' , $user_details->bussiness_id)
            ->update([
                'is_locked' => date('Y-m-d H:i:s', strtotime('+6 hours', strtotime(date('Y-m-d H:i:s')))) ,
                'locked_by' => $user_details->id
            ]);

            // -- audit log start

            $_f_data = FileDocumentModel::select('section_code','name','folder_id')->where('unique_id', $file_uid)->first();

            if(!$_f_data){
                throw new \Exception('File not Found.') ;
            }

            AuditLogService::addAuditLog([
                'file_uid' => $file_uid ,
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $_f_data->folder_id == $_f_data->section_code  ? 0 : $_f_data->folder_id ,
                'action' =>  $_f_data->name . " <b>locked</b>." ,
                'section_code' =>  $_f_data->section_code ,
                'entity_type' => 1,
                'log_type' => 7,
                'text' => 'file locked'
            ]) ;
            // -- audit log end

            return response()->json([
                'status' => true,
                'message' => 'Document Locked Successfully.'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '20',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function unlockFile($file_uid){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            if(!$file_uid || $file_uid == 0){
                throw new \Exception('Invalid file');
            }

            $v_data = FileDocumentModel::where('unique_id', $file_uid)
            ->where('bussiness_id' , $user_details->bussiness_id)
            ->update([
                'is_locked' => null,
                'locked_by' => null
            ]);

            // -- audit log start

            $_f_data = FileDocumentModel::select('section_code','name','folder_id')->where('unique_id', $file_uid)->first();

            if(!$_f_data){
                throw new \Exception('File not Found.') ;
            }

            AuditLogService::addAuditLog([
                'file_uid' => $file_uid ,
                'user_id' =>  $user_details->id,
                'bussiness_id' => $user_details->bussiness_id,
                'folder_id' => $_f_data->folder_id == $_f_data->section_code  ? 0 : $_f_data->folder_id ,
                'action' =>  $_f_data->name . " <b>unlocked</b>." ,
                'section_code' =>  $_f_data->section_code ,
                'entity_type' => 1,
                'log_type' => 17,
                'text' => 'file unlocked'
            ]) ;
            // -- audit log end

            return response()->json([
                'status' => true,
                'message' => 'Document Unlocked Successfully.'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '20',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function saveDocumentComment(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $req = $request->all();

            $validation = Validator::make($req, [
                'file_uid' => "required|numeric|min:1000",
                'comment' => "required|string|min:4"
            ]);

            $validated = HelperFunctionService::getValidationError($validation);

            // echo '<pre>';
            // print_r($validated ) ;
            // exit ;

            DocumentCommentModel::create([
                'file_uid' => $validated['file_uid'] ,
                'comment' => $validated['comment'] ,
                'bussiness_id' => $user_details->bussiness_id,
                'added_by' => $user_details->id
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Comment added successfully'
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '21',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

    public function getDocumentComment($file_uid){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            if(!$file_uid){
                throw new \Exception("File id not valid") ;
            }

            $comments_data = DocumentCommentModel::select('document_comment.created_at', 'document_comment.id', 'document_comment.comment','users.name as user_name','image')
                ->leftJoin('users', 'users.id', '=', 'document_comment.added_by')
                ->where('document_comment.bussiness_id', $user_details->bussiness_id )
                ->where('document_comment.file_uid',$file_uid )
                ->whereNull('document_comment.deleted_at' )
                ->orderBy('id', 'desc')
                ->get()->toArray();

            $comment_ar = [] ;

            foreach($comments_data as $k=>$val){
                $comment_ar[$k] = $val ;

                $comment_ar[$k]['created_at'] = date('d M, Y h:i:s a', strtotime( $val['created_at'] )) ;

                $img_url = null ;
                if($val['image'] ){
                    $img_url = env('APP_URL') .':'.$_SERVER['SERVER_PORT'].'/storage'.$val['image']  ;
                }
                $comment_ar[$k]['img_url'] = $img_url ;

                unset($comment_ar[$k]['image']);
            }

            return response()->json([
                'status' => true,
                'data' => $comment_ar 
            ], 200);

        } catch (\Exception $e) {

            $err = [
                'code' => self::CODE . '22',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info(" ERROR :: ", $err);

            return response()->json([
                'status' => false,
                'message' => $e->getMessage()
            ], 200);
        }
    }

}
