<?php

namespace App\Http\Controllers;

use App\Models\DocumentVersioningModel;
use App\Models\FileDocumentModel;
use App\Models\FolderDirectoryModel;
use App\Models\TemplateFolderAssocModel;
use App\Service\DocDuplicacyService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Service\HelperFunctionService;
use App\Service\TemplateService;
use DateInterval;
use DateTime;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;

class ReportsController extends Controller
{

    protected $templateService;
    protected $duplicacyService;

    public function __construct(TemplateService $templateService, DocDuplicacyService $duplicacyService)
    {
        $this->templateService = $templateService;
        $this->duplicacyService = $duplicacyService;
    }

    public const CODE = 'P' ;
    
    public function versionReportList()
    {
        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;

            $doc_versioning = DocumentVersioningModel::select('document_versioning.*','document_versioning.file_uid as doc_id', 'document_versioning.created_at as added_on','documents.name as version_name', 'documents.folder_id as folder_id', 'folder_directory.folder_name', 'section.section_name', 'users.name as added_by', 'document_versioning.version_number as version'  )
                            ->leftJoin('documents', 'documents.id', '=', 'document_versioning.file_id')
                            ->leftJoin('folder_directory', 'folder_directory.id', '=', 'documents.folder_id')
                            ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                            ->leftJoin('users', 'users.id', '=', 'document_versioning.added_by')
                            ->whereNull('document_versioning.deleted_at')
                            ->where('documents.bussiness_id',$bsns_id)
                            ->whereNull('documents.deleted_at')
                            ->get()->toArray();

            $result = [];
            $i=0;
            $map_ar = []; 
            $m2 = [] ; // for file against version
            foreach($doc_versioning as $val){
                
                $doc_id = $val['doc_id'];
                $versionNumber = $val['version_number'];
                
                if (!isset($map_ar[$doc_id])) {
                    $map_ar[$doc_id] = $i ;
                    $i++;
                }

                $m2[$doc_id][] = $versionNumber ;

                $j = $map_ar[$doc_id] ;

                $k = count( $m2[$doc_id] ) - 1 ;

                $result[$j]['section_name'] = $val['section_name']; 
                $result[$j]['folder_name'] = $val['folder_name']; 
                $result[$j]['folder_name'] = $val['folder_name']; 
                $result[$j]['doc_id'] = $doc_id ;

                $result[$j]['version_data'][$k] = [
                    // 'version_number' => $versionNumber ,
                    'version_number' => $k+1 ,

                    'version_name' => $val['version_name'],
                    'added_by' => $val['added_by'],
                    'added_on' => date('d-m-Y h:i:s a', strtotime($val['added_on']) ) 
                ];                
            }

            return response()->json([
                'status' => true,
                'data' => $result
            ]);

        }catch(\Exception $e){

            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);

        }
    }

    public function retentionReportList()
    {
        try{
            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;

            $ret_documents = FileDocumentModel::select('documents.unique_id as doc_id', 'documents.retention_date as doc_retention_date', 'documents.retention_type  as doc_retention_type', 'documents.retention_period as doc_retention_period', 'documents.folder_id' , 'folder_directory.id', 'folder_directory.folder_name','section.section_name' )
                        ->leftJoin('folder_directory', 'folder_directory.id' , '=', 'documents.folder_id')
                        ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                        ->where('documents.bussiness_id',$bsns_id)
                        ->whereNull('documents.deleted_at')
                        ->whereNotNull('documents.retention_type')
                        ->whereNotNull('documents.unique_id')
                        ->get()->toArray();

            $result = [];

            foreach($ret_documents as $val){

                $retention_date = $val['doc_retention_date'];
                $retention_type = $val['doc_retention_type'];
                $retention_period = $val['doc_retention_period'];

                $future_date = date('Y-m-d', strtotime($retention_date));

                $today = date('Y-m-d');
                $days_left = '';
                $period = [];
                if( $retention_type == 1 && $retention_date != ''){
                    
                    $date1 = new DateTime($today);  
                    $date2 = new DateTime($future_date);   
                    $diff = $date2->diff($date1)->format("%a");  
                    $days_left = intval($diff);

                    $future_timestamp = strtotime($future_date);
                    $today_timestamp = strtotime($today);
                    $diff = ($future_timestamp - $today_timestamp) / (60 * 60 * 24);
                    $days_left = intval($diff);

                }
                elseif( $retention_type == 2 && $retention_period != '' ){

                    $retention_period = $retention_period;

                    $period_values = explode(',', $retention_period);

                    $period = [
                        'days' => isset($period_values[0]) ? (int)$period_values[0] : 0,
                        'months' => isset($period_values[1]) ? (int)$period_values[1] : 0,
                        'years' => isset($period_values[2]) ? (int)$period_values[2] : 0,
                    ];

                    $years = $period['years'];
                    $months = $period['months'];
                    $days = $period['days'];

                    $today_timestamp = strtotime('now');
                    $future_timestamp = strtotime("+$years years", $today_timestamp);
                    $total_days_in_year = floor(($future_timestamp - $today_timestamp) / (60 * 60 * 24));

                    $future_timestamp_month = strtotime("+$months months", $today_timestamp);

                    $total_days_in_month = $months * 30;

                    $total_period = $total_days_in_year + $total_days_in_month + $days;

                    $future_timestamp = strtotime("+$total_period days", $today_timestamp);
                    $future_date = date('Y-m-d', $future_timestamp);

                    $days_left = ($future_timestamp - $today_timestamp) / (60 * 60 * 24);                    
                }
                                
                $result[] = [
                    'doc_id' => $val['doc_id'],
                    'section_name' => $val['section_name'],
                    'folder_name' => $val['folder_name'],
                    'retention_type' => $val['doc_retention_type'],
                    'retention_date' => $val['doc_retention_date'],
                    'retention_period' => $period,
                    'days_left' => $days_left > 0 ? $days_left : 'Already Passed'
                ];          
            }

            return response()->json([
                'status' => true,
                'data' => $result
            ]);


        }catch(\Exception $e){
            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);
        }
    }

    public function exportRetentionReport()
    {
        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;

            $ret_documents = FileDocumentModel::select('documents.unique_id as doc_id', 'documents.retention_date as doc_retention_date', 'documents.retention_type  as doc_retention_type', 'documents.retention_period as doc_retention_period', 'documents.folder_id' , 'folder_directory.id', 'folder_directory.folder_name','section.section_name' )
                        ->leftJoin('folder_directory', 'folder_directory.id' , '=', 'documents.folder_id')
                        ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                        ->where('documents.bussiness_id',$bsns_id)
                        ->whereNull('documents.deleted_at')
                        ->whereNotNull('documents.retention_type')
                        ->whereNotNull('documents.unique_id')
                        ->get()->toArray();

            // -- excel work start

            $spreadsheet = new Spreadsheet();
            $activeWorksheet = $spreadsheet->getActiveSheet();
            $activeWorksheet->setCellValue('A1', 'Document Id');
            $activeWorksheet->setCellValue('B1', 'Section');
            $activeWorksheet->setCellValue('C1', 'Folder Name');
            $activeWorksheet->setCellValue('D1', 'Retention Date');
            $activeWorksheet->setCellValue('E1', 'Retention Period');
            $activeWorksheet->setCellValue('F1', 'Days Left');

            // -- excel work end

            $result = [];
            $i = 2;       

            foreach($ret_documents as $val){

                $retention_date = $val['doc_retention_date'];
                $retention_type = $val['doc_retention_type'];
                $retention_period = $val['doc_retention_period'];

                $future_date = date('Y-m-d', strtotime($retention_date));

                $today = date('Y-m-d');
                $days_left = '';
                $period = [];
                $ret_period = '';
                if( $retention_type == 1 && $retention_date != ''){
                    
                    $date1 = new DateTime($today);  
                    $date2 = new DateTime($future_date);   
                    $diff = $date2->diff($date1)->format("%a");  
                    $days_left = intval($diff);

                    $future_timestamp = strtotime($future_date);
                    $today_timestamp = strtotime($today);
                    $diff = ($future_timestamp - $today_timestamp) / (60 * 60 * 24);
                    $days_left = intval($diff);

                }
                elseif( $retention_type == 2 && $retention_period != '' ){

                    $retention_period = $retention_period;

                    $period_values = explode(',', $retention_period);

                    $period = [
                        'days' => isset($period_values[0]) ? (int)$period_values[0] : 0,
                        'months' => isset($period_values[1]) ? (int)$period_values[1] : 0,
                        'years' => isset($period_values[2]) ? (int)$period_values[2] : 0,
                    ];

                    $years = $period['years'];
                    $months = $period['months'];
                    $days = $period['days'];

                    $today_timestamp = strtotime('now');
                    $future_timestamp = strtotime("+$years years", $today_timestamp);
                    $total_days_in_year = floor(($future_timestamp - $today_timestamp) / (60 * 60 * 24));

                    $future_timestamp_month = strtotime("+$months months", $today_timestamp);

                    $total_days_in_month = $months * 30;

                    $total_period = $total_days_in_year + $total_days_in_month + $days;

                    $future_timestamp = strtotime("+$total_period days", $today_timestamp);
                    $future_date = date('Y-m-d', $future_timestamp);

                    $days_left = ($future_timestamp - $today_timestamp) / (60 * 60 * 24);   
                    
                    $period_days = '';
                    $period_months = '';
                    $period_years = '';
                    if($years>0){
                        $period_years = $years . ' Years ';
                    }
                    if($days > 0){
                        $period_days = $days . ' Days ';
                    }
                    if($months>0){
                        $period_months = $months . ' Months ';
                    }
    
                    $ret_period =  $period_days . $period_months . $period_years ;

                }
                                
                $result[] = [
                    'doc_id' => $val['doc_id'],
                    'section_name' => $val['section_name'],
                    'folder_name' => $val['folder_name'],
                    'retention_type' => $val['doc_retention_type'],
                    'retention_date' => $val['doc_retention_date'],
                    'retention_period' => $period,
                    'days_left' => $days_left > 0 ? $days_left : 'Already Passed'
                ];

                // -- excel work start
                $activeWorksheet->setCellValue("A$i", $val['doc_id'] );
                $activeWorksheet->setCellValue("B$i", $val['section_name'] );
                $activeWorksheet->setCellValue("C$i", $val['folder_name']);
                $activeWorksheet->setCellValue("D$i", $val['doc_retention_date'] );
                $activeWorksheet->setCellValue("E$i", $ret_period );
                $activeWorksheet->setCellValue("F$i", $days_left > 0 ? $days_left.' Days' : 'Already Passed' );
                // -- excel work end

                $i++;
            }

            $writer = new Xlsx($spreadsheet);
            $ts = time() ;
            $f_name = "retention_$ts.xlsx" ;
            $path = "exported/$f_name" ;
            $writer->save($path);

            if(!env("DOWNLOAD_URL") ){
                throw new \Exception('Download url not configured') ;
            }

            $_u = str_replace('storage/','',env("DOWNLOAD_URL")) ;
            $url = $_u . $path ;

            return response()->json([
                'status' => true,
                'data' => [
                    'url' => $url,
                    'file_name' => $f_name,
                ]
            ], 200);

        }catch(\Exception $e){
            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);
        }
    }

    public function templateReportList(Request $request)
    {
        try{

            $documents = FileDocumentModel::select('documents.unique_id as doc_id', 'documents.folder_id', 'documents.template_data', 'documents.template_id', 'template.id', 'template.template_name', 'folder_directory.folder_name', 'section.section_name' )
                        ->leftJoin('template', 'template.id', '=', 'documents.template_id')
                        ->leftJoin('folder_directory', 'folder_directory.id' , '=', 'documents.folder_id')
                        ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                        ->whereNull('documents.deleted_at')
                        ->whereNotNull('documents.unique_id');

            if(isset($request->template_id)){
                $documents = $documents->where('documents.template_id', $request->template_id);
            }

            $documents = $documents->get()->toArray();
           
            $result = [];
            $template_data = '';
            foreach($documents as $val){


                $template_data = json_decode($val['template_data'], true) ;
                $t_data = [];

                $result[] = [
                    'doc_id' => $val['doc_id'],
                    'section_name' => $val['section_name'] ? $val['section_name'] : '-',
                    'folder_name' => $val['folder_name'] ? $val['folder_name'] : '-',
                    'template_id' => $val['template_id'],
                    'template_details' => $template_data ? $template_data : '-'
                ];
                              
            }

            return response()->json([
                'status' => true,
                'data' => $result
            ]);

        }catch(\Exception $e){
            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);
        }
    }

    public function exportTemplateReport(Request $request)
    {
        try{

            $documents = FileDocumentModel::select('documents.unique_id as doc_id', 'documents.folder_id', 'documents.template_data', 'documents.template_id', 'template.id', 'template.template_name', 'folder_directory.folder_name', 'section.section_name' )
                        ->leftJoin('template', 'template.id', '=', 'documents.template_id')
                        ->leftJoin('folder_directory', 'folder_directory.id' , '=', 'documents.folder_id')
                        ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                        ->whereNull('documents.deleted_at')
                        ->whereNotNull('documents.unique_id');

            if(isset($request->template_id)){
                $documents = $documents->where('documents.template_id', $request->template_id);
            }

            // -- excel work start

            $spreadsheet = new Spreadsheet();
            $activeWorksheet = $spreadsheet->getActiveSheet();
            $activeWorksheet->setCellValue('A1', 'Document Id');
            $activeWorksheet->setCellValue('B1', 'Section');
            $activeWorksheet->setCellValue('C1', 'Folder Name');
            $activeWorksheet->setCellValue('D1', 'Template Details');

            // -- excel work end

            $documents = $documents->get()->toArray();
           
            $template_data = '';
            $i = 2;
            foreach($documents as $val){


                $template_data = json_decode($val['template_data'], true) ;

                $t_data = '';
                if($template_data){

                    foreach($template_data as $key => $v){                  
                        $t_data .= $key . " : " . $v. " \n ";
                    }
                }
            
                // -- excel work start
                $activeWorksheet->setCellValue("A$i", $val['doc_id'] );
                $activeWorksheet->setCellValue("B$i", $val['section_name'] );
                $activeWorksheet->setCellValue("C$i", $val['folder_name']);
                $activeWorksheet->setCellValue("E$i", $t_data );
                // -- excel work end

                $i++;
                              
            }

            $writer = new Xlsx($spreadsheet);
            $ts = time() ;
            $f_name = "template_$ts.xlsx" ;
            $path = "exported/$f_name" ;
            $writer->save($path);

            if(!env("DOWNLOAD_URL") ){
                throw new \Exception('Download url not configured') ;
            }

            $_u = str_replace('storage/','',env("DOWNLOAD_URL")) ;
            $url = $_u . $path ;

            return response()->json([
                'status' => true,
                'data' => [
                    'url' => $url,
                    'file_name' => $f_name,
                ]
            ], 200);

        }catch(\Exception $e){
            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);
        }
    }

    public function exportVersionReport()
    {
        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $bsns_id = $user_details->bussiness_id ;            

            $doc_versioning = DocumentVersioningModel::select('document_versioning.*','document_versioning.file_uid as doc_id', 'document_versioning.created_at as added_on','documents.name as version_name', 'documents.folder_id as folder_id', 'folder_directory.folder_name', 'section.section_name', 'users.name as added_by', 'document_versioning.version_number as version'  )
                            ->leftJoin('documents', 'documents.id', '=', 'document_versioning.file_id')
                            ->leftJoin('folder_directory', 'folder_directory.id', '=', 'documents.folder_id')
                            ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                            ->leftJoin('users', 'users.id', '=', 'document_versioning.added_by')
                            ->whereNull('document_versioning.deleted_at')
                            ->where('documents.bussiness_id',$bsns_id)
                            ->whereNull('documents.deleted_at')
                            ->get()->toArray();

            $result = [];
            $i=0;
            $map_ar = []; 
            $m2 = [] ; // for file against version

            // -- excel work start

            $spreadsheet = new Spreadsheet();
            $activeWorksheet = $spreadsheet->getActiveSheet();
            $activeWorksheet->setCellValue('A1', 'Document Id');
            $activeWorksheet->setCellValue('B1', 'Section');
            $activeWorksheet->setCellValue('C1', 'Folder Name');
            $activeWorksheet->setCellValue('D1', 'Version Number');
            $activeWorksheet->setCellValue('E1', 'Document Name');
            $activeWorksheet->setCellValue('F1', 'Added By');
            $activeWorksheet->setCellValue('G1', 'Uploaded On');


            // -- excel work end

            $result = [];
            $i=0;
            $map_ar = []; 
            $m2 = [] ; // for file against version
            foreach($doc_versioning as $val){
                
                $doc_id = $val['doc_id'];
                $versionNumber = $val['version_number'];
                
                if (!isset($map_ar[$doc_id])) {
                    $map_ar[$doc_id] = $i ;
                    $i++;
                }

                $m2[$doc_id][] = $versionNumber ;

                $j = $map_ar[$doc_id] ;

                $k = count( $m2[$doc_id] ) - 1 ;

                $result[$j]['section_name'] = $val['section_name']; 
                $result[$j]['folder_name'] = $val['folder_name']; 
                $result[$j]['folder_name'] = $val['folder_name']; 
                $result[$j]['doc_id'] = $doc_id ;

                $result[$j]['version_data'][$k] = [
                    // 'version_number' => $versionNumber ,
                    'version_number' => $k+1 ,

                    'version_name' => $val['version_name'],
                    'added_by' => $val['added_by'],
                    'added_on' => date('d-m-Y h:i:s a', strtotime($val['added_on']) ) 
                ];                
            }

            $o = 2;
            foreach($result as $val){
                $activeWorksheet->setCellValue("A$o", $val['doc_id'] );
                $activeWorksheet->setCellValue("B$o", $val['section_name'] );
                $activeWorksheet->setCellValue("C$o", $val['folder_name'] );

                $m = 0 ; 
                foreach( $val['version_data'] as  $v2 ){

                    if($m!=0){
                        $activeWorksheet->setCellValue("A$o", $val['doc_id'] );
                        $activeWorksheet->setCellValue("B$o", $val['section_name'] );
                        $activeWorksheet->setCellValue("C$o", $val['folder_name'] );
                        $m=1;
                        $o++;
                    }

                    $activeWorksheet->setCellValue("D$o", $v2['version_number'] );
                    $activeWorksheet->setCellValue("E$o", $v2['version_name'] );
                    $activeWorksheet->setCellValue("F$o", $v2['added_by'] );
                    $activeWorksheet->setCellValue("G$o", $v2['added_on'] );
                    $o++;
                }

                $o++;
            }

            $reader = new \PhpOffice\PhpSpreadsheet\Reader\Html();

            $writer = new Xlsx($spreadsheet);
            $ts = time() ;
            $f_name = "version_$ts.xlsx" ;
            $path = "exported/$f_name" ;
            $writer->save($path);

            if(!env("DOWNLOAD_URL") ){
                throw new \Exception('Download url not configured') ;
            }

            $_u = str_replace('storage/','',env("DOWNLOAD_URL")) ;
            $url = $_u . $path ;

            return response()->json([
                'status' => true,
                'data' => [
                    'url' => $url,
                    'file_name' => $f_name,
                ]
            ], 200);

        }catch(\Exception $e){

            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);

        }
    }
    
    public function duplicateReport()
    {
        try{

            $documents = FileDocumentModel::select('documents.unique_id as doc_id', 'documents.name', 'documents.path', 'documents.folder_id', 'documents.section_code' , 'folder_directory.folder_name', 'folder_directory.parent_folder_id','section.section_name' )
                                            ->leftJoin('folder_directory', 'folder_directory.id', '=', 'documents.folder_id')
                                            ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                                            ->whereNull('documents.deleted_at')
                                            ->whereNotNull('documents.unique_id')
                                            ->orderBy('documents.name')
                                            ->get()
                                            ->toArray(); 

            $documentNameCounts = [];
            foreach ($documents as $value) {
                $documentName = $value['name'];
                $documentNameCounts[$documentName] = isset($documentNameCounts[$documentName]) ? $documentNameCounts[$documentName] + 1 : 1;
            }
            $result = [];

            foreach($documents as $value){
              
                $doc_path = '';
                if( $value['parent_folder_id'] == 0 || $value['parent_folder_id'] == ''  ){
                    $doc_path = $value['folder_name'];
                }
                else{
                    $doc_path = $this->duplicacyService->buildDocumentPath($value['parent_folder_id']) . $value['folder_name'];
                }

                $documentName = $value['name'];

                if ($documentNameCounts[$documentName] > 1) {
                    $result[] = [
                        'doc_id' => $value['doc_id'],
                        'section_name' => $value['section_name'] ? $value['section_name'] : '-',
                        'folder_name' => $value['folder_name'] ? $value['folder_name'] : '-',
                        'document_name' => $documentName ? $documentName : '-',
                        'document_path' => $doc_path ? $doc_path : '-',
                    ];
                }
            }

            return response()->json([
                'status' => true,
                'data' => $result
            ]);

        }catch(\Exception $e){
            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);

        }
    }

    public function exportDuplicacyReport()
    {
        try{

            $documents = FileDocumentModel::select('documents.unique_id as doc_id', 'documents.name', 'documents.path', 'documents.folder_id', 'documents.section_code' , 'folder_directory.folder_name', 'folder_directory.parent_folder_id','section.section_name' )
                                            ->leftJoin('folder_directory', 'folder_directory.id', '=', 'documents.folder_id')
                                            ->leftJoin('section', 'section.section_code', '=', 'documents.section_code')
                                            ->whereNull('documents.deleted_at')
                                            ->whereNotNull('documents.unique_id')
                                            ->orderBy('documents.name')
                                            ->get()
                                            ->toArray();
            
            
            $documentNameCounts = [];
            foreach ($documents as $value) {
                $documentName = $value['name'];
                $documentNameCounts[$documentName] = isset($documentNameCounts[$documentName]) ? $documentNameCounts[$documentName] + 1 : 1;
            }

            // -- excel work start
            $spreadsheet = new Spreadsheet();
            $activeWorksheet = $spreadsheet->getActiveSheet();
            $activeWorksheet->setCellValue('A1', 'Document Id');
            $activeWorksheet->setCellValue('B1', 'Section');
            $activeWorksheet->setCellValue('C1', 'Folder');
            $activeWorksheet->setCellValue('D1', 'Document Name');
            $activeWorksheet->setCellValue('E1', 'Document Path');


            // -- excel work end
            $j = 2;
            foreach($documents as $value){
              
                $doc_path = '';
                if( $value['parent_folder_id'] == 0 || $value['parent_folder_id'] == ''  ){
                    $doc_path = $value['folder_name'];
                }
                else{
                    $doc_path = $this->duplicacyService->buildDocumentPath($value['parent_folder_id']) . $value['folder_name'];
                }   
                
                $documentName = $value['name'];

                if ($documentNameCounts[$documentName] > 1) {
                    $activeWorksheet->setCellValue("A$j", $value['doc_id'] );
                    $activeWorksheet->setCellValue("B$j", $value['section_name'] ? $value['section_name'] : '-' );
                    $activeWorksheet->setCellValue("C$j", $value['folder_name'] ? $value['folder_name'] : '-' );
                    $activeWorksheet->setCellValue("D$j", $documentName ? $documentName : '-');
                    $activeWorksheet->setCellValue("E$j", $doc_path ? $doc_path : '-' );
                }



                $j++;
            }

            $writer = new Xlsx($spreadsheet);
            $ts = time() ;
            $f_name = "duplicacy_$ts.xlsx" ;
            $path = "exported/$f_name" ;
            $writer->save($path);

            if(!env("DOWNLOAD_URL") ){
                throw new \Exception('Download url not configured') ;
            }

            $_u = str_replace('storage/','',env("DOWNLOAD_URL")) ;
            $url = $_u . $path ;

            return response()->json([
                'status' => true,
                'data' => [
                    'url' => $url,
                    'file_name' => $f_name,
                ]
            ], 200);

        }catch(\Exception $e){
            $err = [
                'code' => self::CODE.'01',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];
 
             Log::error( " ERROR :: " , $err ) ;
 
             return response()->json([
                 'status'=>'false',
                 'message'=> $e->getMessage()
             ],200);

        }
    }

}
