<?php

namespace App\Http\Controllers;

use App\Models\VerifyOtpModel;
use App\Models\User;
use App\Models\BussinessModel;
use Illuminate\Support\Facades\Validator;
use App\Service\CommonService;
use App\Service\HelperFunctionService;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use App\Mail\SendOtp;
use App\Mail\SuccessfullRegistration;
use Illuminate\Support\Facades\Mail;


class SuperadminController extends Controller
{

    public const CODE = 'SC' ;

    // not in use
    public function getOnBoarderUser( Request $request){

        try{
            $user_details = HelperFunctionService::doValidateForUser() ;

            $bsns_id = $user_details->bussiness_id ;

            $user_d = User::select('id','name', 'email')->where('bussiness_id',$bsns_id)->where('id','!=',$user_details->id)
            ->where('role_id',3)
            ->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            return response()->json([
                'status' => true,
                'data' => $user_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function registerBussiness(Request $request){

        try{

            $input = $request->all();

            $validation = Validator::make($input,[
                'name'=>'required|string',
                'phone'=>'required|digits_between:10,15',
                'email'=>'required|email', // |unique:users,email
                'company_name' => 'required|nullable',
                'gst' => 'nullable'
            ]);

            if($validation->fails()){
                $errorMessages = $validation->errors();

                $formattedErrors = [];
                $msg = '' ;

                $cnt = count($errorMessages->messages()) ;
                foreach ($errorMessages->messages() as $key => $messages) {
                    $formattedErrors[$key] = $messages[0];
                    $msg .= $formattedErrors[$key] . '<br>'  ;
                }

                return response()->json([
                    'status' => false,
                    'message' => $msg
                ], 200);
            }

            $validator = $validation->validated();

            // -- check email uniq strt

            $user_d = User::select('id','is_email_verify','email_verified_at','bussiness_id')
                ->where('email',$validator['email'] )
                ->whereNull('deleted_at')
                ->first();

            if($user_d){
                if( $user_d->is_email_verify == 1){
                    return response()->json([
                        'status' => false,
                        'message' => 'Email already taken'
                    ], 200);
                }

                if($user_d->bussiness_id){
                    BussinessModel::where('id', $user_d->bussiness_id )->update(['deleted_at'=>date('Y-m-d H:i:s') ]);
                }

                User::where('id', $user_d->id )->update(['deleted_at'=>date('Y-m-d H:i:s') ]);
            }

            // -- check email uniq end

            $ins_bsns = BussinessModel::create([
                'company_name' => $validator['company_name'] ,
                'current_plan' => 1, // basic plan
            ]);

            $bsns_id = $ins_bsns->id;

            User::insert([
                'name' => $validator['name'] ,
                'email' => $validator['email'] ,
                'phone' => $validator['phone'] ,
                'role_id' => 2 ,
                'bussiness_id' => $bsns_id
            ]) ;

            $otp = CommonService::getOTP(['length'=>6]) ;

            Mail::to($validator['email'] )->send(new SendOtp( ['otp'=>$otp ] ));

            VerifyOtpModel::create([
                'email_id' => $validator['email'] ,
                'otp' => $otp
            ]);

            return response()->json([
                'status' => true,
                'message' => "Thanks for the registration. Please verify your email."
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'14',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function verifyOTP(Request $request){

        try{

            $input = $request->all();

            $validation = Validator::make($input,[
                'otp'=>'required|numeric',
                'email_id'=>'required|email'
            ]);

            if($validation->fails()){
                $errorMessages = $validation->errors();

                $formattedErrors = [];
                $msg = '' ;

                $cnt = count($errorMessages->messages()) ;
                foreach ($errorMessages->messages() as $key => $messages) {
                    $formattedErrors[$key] = $messages[0];
                    $msg .= $formattedErrors[$key] . '<br>'  ;
                }

                return response()->json([
                    'status' => false,
                    'message' => $msg
                ], 200);
            }

            $validator = $validation->validated();

            $_d = VerifyOtpModel::select('otp','id')
                ->where('email_id',$validator['email_id'] )
                ->where('is_verify', 0 )
                ->orderBy('id', 'desc')
                ->limit(1)
                ->first();

            if(!$_d){
                return response()->json([
                    'status' => false,
                    'message' => 'Email Not Verified.'
                ],200);
            }

            $ar = $_d->toArray() ;

            if( $ar['otp'] == $validator['otp']){

                VerifyOtpModel::where('id',$ar['id'] )->update([
                    'is_verify' => 1
                ]);

                $pasw = CommonService::generatePassword() ;

                User::where('email', $validator['email_id'] )->update([
                    'password' => bcrypt($pasw) ,
                    'is_email_verify' => 1 ,
                    'email_verified_at' => date('Y-m-d H:i:s')
                ]) ;

                Mail::to($validator['email_id'] )->send(new SuccessfullRegistration( ['pasw'=>$pasw , 'email_id' => $validator['email_id'] ] ));

                return response()->json([
                    'status' => true,
                    'message' => 'Email Verified Successfully.'
                ],200);
            }

            return response()->json([
                'status' => false,
                'message' => 'Email Not Verified.'
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'13',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function getRegUsers(Request $request){

        try{

            $plan_ar = config('plan.plan.plan_name') ;

            $registered_user_ar = User::select( 'users.id as uid', 'bussiness.id as bid' , 'bussiness.plan_expired_on' ,'users.name', 'users.email' ,'users.is_email_verify', 'users.email_verified_at' , 'users.phone' ,'bussiness.company_name' , 'bussiness.created_at', 'current_plan' )
            ->leftJoin('bussiness', 'users.bussiness_id', '=', 'bussiness.id')
            ->where('role_id', 2)
            ->whereNull('bussiness.deleted_at')
            ->orderBy('bussiness.id','desc')
            ->get()->toArray() ;

            $list = [];

            foreach($registered_user_ar as $k=> $val ){
                $list[$k] = $val ;
                $list[$k]['plan_name'] = isset($plan_ar[$val['current_plan']]) ? $plan_ar[$val['current_plan']] : '' ;
                $list[$k]['created_at'] = $val['created_at'] ? date('d M, Y | h:i:s a', strtotime( $val['created_at'] )) : null ;
                $list[$k]['plan_expired_on'] = $val['plan_expired_on'] ? date('d M, Y', strtotime( $val['plan_expired_on'] )) : null ;
            }

            return response()->json([
                'status' => true,
                'data' => $list
            ],200);


        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'12',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function resendOTP(Request $request){

        try{

            $input = $request->all();

            $validation = Validator::make($input,[
                'email_id'=>'required|email',
            ]);

            if($validation->fails()){
                $errorMessages = $validation->errors();

                $formattedErrors = [];
                $msg = '' ;

                $cnt = count($errorMessages->messages()) ;
                foreach ($errorMessages->messages() as $key => $messages) {
                    $formattedErrors[$key] = $messages[0];
                    $msg .= $formattedErrors[$key] . '<br>'  ;
                }

                return response()->json([
                    'status' => false,
                    'message' => $msg
                ], 200);
            }

            $validator = $validation->validated();

            // $bsns_id = $ins_bsns->id;

            $otp = CommonService::getOTP(['length'=>6]) ;

            Mail::to($validator['email_id'] )->send(new SendOtp( ['otp'=>$otp ] ));

            VerifyOtpModel::create([
                'email_id' => $validator['email_id'] ,
                'otp' => $otp
            ]);

            return response()->json([
                'status' => true,
                'message' => "Email Resend Successfully."
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'11',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
            ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

}
