<?php

namespace App\Http\Controllers;

use App\Models\TemplateFolderAssocModel;
use App\Models\TemplateModel;
use App\Models\TemplateUserAssocModel;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use App\Service\HelperFunctionService;
use App\Service\FileDetailService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class TemplateController extends Controller
{

    public const CODE = 'T' ;

    public function templateList(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'user_id'=>'nullable|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $template_d = TemplateModel::select()->where('bussiness_id',$bsns_id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            foreach($template_d as $k=>$val){
                $template_d[$k]['created_at'] = date('M d,Y', strtotime($val['created_at']) );
            }

            return response()->json([
                'status' => true,
                'data' => $template_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }



    /**
     * add and edit template
     */
    public function addTemplate( Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'fields.template_name'=>'required|regex:/^[a-zA-Z0-9 -]*$/', // alpha numeric with space and hypen are allowed
                'fields' => 'required',
                'i' => 'required',
                'is_edit' => 'required',
                'template_id' => 'nullable',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $fields = $req['fields'] ;

            unset($fields['template_name']);

            if( $req['is_edit'] ){

                $template_id = TemplateModel::where('id',$req['template_id'] )->update([
                    'template_name' => $req['fields']['template_name'] ,
                    'i_index'=> json_encode( $req['i'] ) ,
                    'fields'=> json_encode( $fields ),
                ]);

                return response()->json([
                    'status' => true,
                    'message' => 'Template updated successfully.',
                ],200);
            }

            $template_id = TemplateModel::create([
                'template_name' => $req['fields']['template_name'] ,
                'i_index'=> json_encode( $req['i'] ) ,
                'fields'=> json_encode( $fields ),
                'bussiness_id' => $bsns_id,
                'added_by' => $user_details->id,
            ])->id;

            return response()->json([
                'status' => true,
                'message' => 'Template added successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'02',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }
    public function deleteTemplate(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'template_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            TemplateModel::where('id',$req['template_id'])->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Template Deleted successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'03',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function templateDetail(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'template_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $template_d = TemplateModel::select('template_name',
            'fields',
            'i_index')->where('id',$req['template_id'])->where('added_by',$user_details->id)->whereNull('deleted_at')->first();

            if(!$template_d){
                throw new \Exception('Template not found') ;
            }

            return response()->json([
                'status' => true,
                'data' => $template_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'06',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function templateListForAssoc (Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'folder_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $_data = TemplateFolderAssocModel::select('template_id','is_apply_on_sub_folder')->where('folder_id',$request->folder_id)->where('bussiness_id',$bsns_id )->first();

            $template_d = TemplateModel::select( 'id','template_name' , 'fields' , 'i_index' )->where('bussiness_id',$bsns_id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            $template_d2 = [] ; 
            $_ar = [] ;
            foreach($template_d as $k => $val){
                $temp_detail = FileDetailService::getDynamicForm($val);

                unset($val['fields']);
                unset($val['i_index']);

                $template_d2[$k] = $val ;
                if($temp_detail['status']){
                    $template_d2[$k]['form_ar'] = $temp_detail['form_ar'] ;
                }
            }   

            /**
             * label
             * type
             * mandatory
             */

            return response()->json([
                'status' => true,
                'data' => $template_d2,
                'selected_temp_id' => $_data ? $_data->template_id : null,
                'is_apply_on_sub_folder' => $_data ? $_data->is_apply_on_sub_folder : null,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'08',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function saveTemplateFolderAssoc(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForUser() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'template_id'=>'required|numeric',
                'folder_id'=>'numeric|required',
                'is_apply_to_sub_folder'=>'numeric|required',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $count = TemplateFolderAssocModel::where('folder_id',$request->folder_id)->where('bussiness_id',$bsns_id )->count();

            if($count==0){ // create

                TemplateFolderAssocModel::create([
                    'template_id' => $request->template_id,
                    'folder_id' => $request->folder_id,
                    'added_by' => $user_details->id,
                    'is_apply_on_sub_folder' => $request->is_apply_to_sub_folder ? 1 : 0,
                    'bussiness_id' => $bsns_id
                ]) ;

            }else{ // update

                TemplateFolderAssocModel::where('folder_id',$request->folder_id)->update([
                    'template_id' => $request->template_id,
                    'added_by' => $user_details->id,
                    'is_apply_on_sub_folder' => $request->is_apply_to_sub_folder ? 1 : 0,
                    'bussiness_id' => $bsns_id
                ]) ;
            }

            return response()->json([
                'status' => true,
                'message' => 'Data updated successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'07',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

}
