<?php

namespace App\Http\Controllers;
use Illuminate\Support\Facades\Validator;
use App\Models\User;
use App\Models\BussinessModel;
use App\Service\HelperFunctionService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
// use Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Throwable;
use App\Mail\SendOtp;
use Illuminate\Support\Facades\Mail;
use App\Service\AuditLogService;

class UserController extends Controller
{
    public const CODE = 'B' ;

    /**
     * Author : Bharat
     */
    public function userLogin(Request $request)
    {
        try {

            $input = $request->all();

            $validation = Validator::make($input,[
                'email'=>"required|email",
                'password'=>"required"
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $credentials = [
                'email' => $request->email ,
                'password' => $request->password ,
            ];

            $user = User::where('email', $request->email)->first();

            if ( $user && Auth::attempt($credentials)) {

                $_user = Auth::user();

                $img_url = null ;
                if($_user->image){
                    $img_url = env('APP_URL') .':'.$_SERVER['SERVER_PORT'].'/storage'.$_user->image ;
                }

                $user_det = [
                    'user_id' => $_user->id,
                    'bussiness_id' => $_user->bussiness_id,
                    'role' => $_user->role_id,
                    'img_url' => $img_url,
                    'user_name' => $_user->name
                ];

                User::where('id',$_user->id)->update( ['last_login_at' => date('Y-m-d H:i:s')]) ;

                $token = $user->createToken('API Token')->accessToken ;

                //-- audit log start
                AuditLogService::addAuditLog([
                    'user_id' =>  $_user->id,
                    'bussiness_id' => $_user->bussiness_id,
                    'action' =>   '<b>Logged in</b>' ,
                    'entity_type' => 3,
                    'log_type' => 21,
                    'text' => 'user login'
                ]) ;
                //-- audit log end

                return response([ 'status' => true ,'token' => $token, 'user_details' => $user_det ]);
            }else{
                throw new \Exception('Invalid Credentials') ;
            }

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status'=>'false',
                'message'=> $e->getMessage()
            ],200);
       }

    }

    /**
     * Author : Bharat
     */
    public function addUser(Request $request)
    {
        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $input = $request->all();

            $validation = Validator::make($input,[
                'username'=>'required|max:204',
                'phone'=>'nullable|digits_between:10,15',
                'email'=>'required|email',
                'is_edit'=>'required',
                'user_id'=>'nullable',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            // -------- check plan
            $bsns_id = $user_details->bussiness_id ;
            $bsn_data = BussinessModel::select('current_plan')->where('id',$bsns_id)->first() ;

            $plan_ar = config('plan.plan.plan_type') ;

            if(!isset( $plan_ar[$bsn_data->current_plan])){
                throw new \Exception('Invalid Plan') ;
            }

            $total_added_users = User::whereNull('deleted_at')->where('bussiness_id',$bsns_id)->where('role_id',3)->count();

            $total_allowed_users = $plan_ar[$bsn_data->current_plan]['users'] ;

            if($total_added_users >= $total_allowed_users){
                throw new \Exception('Your Plan Reached the limit of Maximum Users. Please Buy Users for adding more users.') ;
            }
            // -------- check plan

            if( $input['is_edit']==1 && $input['user_id']!=0 ){

                $file_path = '' ;

                if($request->file('file')) {
                    $fileName = time().'_'. str_replace(" ","_",  $request->file->getClientOriginalName() );
                    $filePath = $request->file('file')->storeAs('user_image', $fileName, 'public');
                    $file_path = '/user_image/' . $fileName;
                }

                User::where('id',$input['user_id'])->update([
                    'name' => $input['username'],
                    'email' => $input['email'],
                    'phone' => $input['phone'],
                    'image' => $file_path,
                ]);

                return response()->json([
                    'status'=>true,
                    'message'=>"User Updated Successfully."
                ],200);
            }

            // --- check email must unique strt
            $user_d = User::select('id')->where('bussiness_id',$user_details->bussiness_id)->where('email',$request->email )->first();
            if($user_d){
                throw new \Exception('Email is already exist.') ;
            }
            // --- check email must unique ends

            $user= new User;
            $user->name = $request->username;
            $user->email = $request->email;
            $user->phone = $request->phone;
            $user->bussiness_id = $user_details->bussiness_id;
            $user->password = '$2y$10$.U/iwPxhCG0onUba9cE/g.K9D2.Ce9mdVNeLAA0ku7ksBiMYcUlO2';

            $user->role_id = 3 ; // 1 superadmin , 2 group admin , 3 simple user
            // $user->password = $request->password;

            $file_path = '' ;

            if($request->file('file')) {
                $fileName = time().'_'. str_replace(" ","_",  $request->file->getClientOriginalName() );
                $filePath = $request->file('file')->storeAs('user_image', $fileName, 'public');
                $file_path = '/user_image/' . $fileName;
            }

            $user->image = $file_path ;

            $res = $user->save();

            if($res){
                return response()->json([
                    'status'=>true,
                    'message'=>"User Created Successfully."
                ],200);
            }

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'01',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function userList(Request $request){

        try{

            $user_details = HelperFunctionService::doValidateForGroupAdmin() ;

            $req = $request->all();

            $validation = Validator::make($req,[
                'user_id'=>'nullable|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $user_d = User::select()->where('bussiness_id',$bsns_id)
            ->where('role_id',3)
            ->where('id','!=',$user_details->id)->orderBy('id','desc')->whereNull('deleted_at')->get()->toArray();

            $user_d_ = [] ;
            foreach($user_d as $k=>$val){
                $user_d[$k]['created_at'] = date('M d,Y', strtotime($val['created_at']) );
                if($val['image']){
                    $user_d[$k]['img_url'] = env('APP_URL') .':'.$_SERVER['SERVER_PORT'].'/storage'.$val['image'];
                }else{
                    $user_d[$k]['img_url'] = null;
                }
            }

            return response()->json([
                'status' => true,
                'data' => $user_d,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'04',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }


    public function userDetail(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'user_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $bsns_id = $user_details->bussiness_id ;

            $user_d = User::select()->where('bussiness_id',$bsns_id)->whereNull('deleted_at')->where('id','=',$request->user_id)->first();

            if(!$user_d){
                throw new \Exception('User not found') ;
            }

            $user_det = $user_d->toArray() ;

            $user_det['created_at'] = date('M d,Y', strtotime($user_det['created_at']) );
            if($user_det['image']){
                $user_det['img_url'] = env('APP_URL') .':'.$_SERVER['SERVER_PORT'].'/storage'.$user_det['image'];
            }else{
                $user_det['img_url'] = null;
            }

            return response()->json([
                'status' => true,
                'data' => $user_det,
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'05',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function userDelete(Request $request){

        try{

            $user_details = Auth::user();

            if(!$user_details){
                throw new \Exception('User is not authenticate');
            }

            if($user_details->role_id!=2){
                throw new \Exception('Permission denied');
            }

            $req = $request->all();

            $validation = Validator::make($req,[
                'user_id'=>'required|numeric',
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            User::where('id',$req['user_id'])->update([
                'deleted_at' => date('Y-m-d H:i:s')
            ]);

            return response()->json([
                'status' => true,
                'message' => 'User Deleted successfully.',
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'06',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function getUserDet(){

        try{

            $user_details = HelperFunctionService::doValidateForUser();

            $user_d = User::select()->where('id',$user_details->id)->whereNull('deleted_at')->first();

            $img_url = null ;

            if($user_d->image ){
                $img_url = env('APP_URL') .':'.$_SERVER['SERVER_PORT'].'/storage'.$user_d->image  ;
            }

            return response()->json([
                'status'=>true,
                'data'=>$user_d,
                'img_url' => $img_url
            ],200);


        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'07',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function updateProfile(Request $request)
    {
        try{

            $user_details = HelperFunctionService::doValidateDashboardUser() ;

            $input = $request->all();

            $validation = Validator::make($input,[
                'username'=>'required|max:204',
                'phone'=>'nullable|digits_between:10,15',
                'email'=>'required|email',
                'user_old_pasw' => 'nullable',
                'user_new_pass' => 'nullable',
                'user_cnfrm_pass' => 'nullable'
            ]);

            if($validation->fails()){
                return response()->json([
                    'status'=>false,
                    'message'=>$validation->errors()
                ],200);
            }

            $validator = $validation->validated();

            if($validator['user_old_pasw']){

                $u_data = User::select('password')->where('id', $user_details->id )->first() ;

                if(!$u_data){
                    throw new \Exception('Invalid User');
                }

                $password = $u_data->password ;

                if (!password_verify($validator['user_old_pasw'], $password)) {
                    throw new \Exception('Invalid Old Password') ;
                }

                if($validator['user_new_pass']==''){
                    throw new \Exception('New Password can not be blank') ;
                }

                if($validator['user_new_pass']!=$validator['user_cnfrm_pass']){
                    throw new \Exception('New Password and confirm password is not same') ;
                }
            }

            $file_path = '' ;

            if($request->file('file')) {
                $fileName = time().'_'. str_replace(" ","_",  $request->file->getClientOriginalName() );
                $filePath = $request->file('file')->storeAs('user_image', $fileName, 'public');
                $file_path = '/user_image/' . $fileName;
            }

            $upd_ar = [
                'name' => $input['username'],
                // 'email' => $input['email'],
                'phone' => $input['phone'],
            ] ;

            if($validator['user_new_pass']!=''){
                $upd_ar['password'] = bcrypt($validator['user_new_pass'])  ;
            }

            if($file_path!=''){
                $upd_ar['image'] = $file_path ;
            }

            User::where('id', $user_details->id )->update($upd_ar);

            return response()->json([
                'status' => true,
                'message' => "Profile Updated Successfully."
            ],200);

        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'12',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }

    public function userLogout(){

        try{




        }catch(\Exception $e){

            $err = [
               'code' => self::CODE.'13',
               'message' => $e->getMessage(),
               'file' => $e->getFile(),
               'line' => $e->getLine()
           ];

            Log::info( " ERROR :: " , $err ) ;

            return response()->json([
                'status' => false,
                'message'=> $e->getMessage()
            ],200);
       }
    }
}
