<?php

namespace App\Service;

use App\Models\AuditLogModel;
use App\Models\FileDocumentModel;
use App\Models\FolderDirectoryModel;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;


class MultiService
{
    // check of delete level missing
    public function delRecursive(array $ar ){

        $folder_id = $ar['folder_id'] ;
        $bsns_id = $ar['bussiness_id'] ;
        
        FileDocumentModel::where('folder_id', $folder_id )
        ->where('bussiness_id', $bsns_id)
        ->whereNull('deleted_at')
        ->update([
            'deleted_at' => date('Y-m-d H:i:s') 
        ]);

        Log::info('MS001 :: file del where folder id :  '. $folder_id );

        $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')->where('parent_folder_id', $folder_id )
        ->whereNull('deleted_at')
        ->get()->toArray() ;

        $is_upd = FolderDirectoryModel::where('parent_folder_id',$folder_id)
        ->where('bussiness_id', $bsns_id)
        ->whereNull('deleted_at')
        ->update([
            'deleted_at' => date('Y-m-d H:i:s') 
        ]);

        foreach($folder_data as $val){
            $this->delRecursive(['folder_id'=>$val['id'], 'bussiness_id'=>$bsns_id ]) ;
            Log::info('MS002 :: folder del :  '. $val['id']);
        }
    }

    // check of delete level missing
    public function recoverRecursive(array $ar ){

        $folder_id = $ar['folder_id'] ;
        $bsns_id = $ar['bussiness_id'] ;
        // $delete_level = isset($ar['delete_level']) ? $ar['delete_level'] : 0 ;

        FileDocumentModel::where('folder_id', $folder_id )
        ->where('bussiness_id', $bsns_id)
        ->where('delete_level', 0)
        ->update([
            'deleted_at' => null
        ]);

        $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')->where('parent_folder_id', $folder_id )
        ->where('delete_level', 0)
        ->whereNotNull('deleted_at')
        ->get()->toArray() ;

        $is_upd = FolderDirectoryModel::where('parent_folder_id',$folder_id )
        ->where('bussiness_id', $bsns_id)
        ->where('delete_level', 0)
        ->update([
            'deleted_at' => null
        ]);

        foreach($folder_data as $val){
            $this->recoverRecursive(['folder_id'=>$val['id'], 'bussiness_id'=>$bsns_id ]) ;
        }
    }

    public function delPermRecursive(array $ar ){

        $folder_id = $ar['folder_id'] ;
        $bsns_id = $ar['bussiness_id'] ;
        
        FileDocumentModel::where('folder_id', $folder_id )
        ->where('bussiness_id', $bsns_id)
        // ->whereNotNull('deleted_at')
        ->update([
            'delete_level' => 1
        ]);

        Log::info('MS004 :: file del permanent where folder id :  '. $folder_id );

        $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')->where('parent_folder_id', $folder_id )
        ->whereNull('deleted_at')
        ->get()->toArray() ;

        $is_upd = FolderDirectoryModel::where('parent_folder_id',$folder_id)
        ->where('bussiness_id', $bsns_id)
        // ->whereNotNull('deleted_at')
        ->update([
            'delete_level' => 1
        ]);

        foreach($folder_data as $val){
            $this->delPermRecursive(['folder_id'=>$val['id'], 'bussiness_id'=>$bsns_id ]) ;
            Log::info('MS002 :: folder del :  '. $val['id']);
        }
    }

    public function downloadFolderRecursive(array $ar ){

        $folder_id = $ar['folder_id'] ;
        $bsns_id = $ar['bussiness_id'] ;
        $destination_pre_path = $ar['destination_pre_path'] ;

        $folder_data = FolderDirectoryModel::select('folder_name')
                ->where('bussiness_id', $bsns_id)
                ->whereNull('deleted_at')
                ->where('id', $folder_id)->first() ;

        $get_uploaded_files = FileDocumentModel::select('path','name')
            ->whereNull('deleted_at')
            ->where('bussiness_id', $bsns_id)
            ->where('folder_id', $folder_id)
            ->get()->toArray();

        $destination_pre_path_2 = $destination_pre_path.$folder_data->folder_name ;
            
        try{
            File::makeDirectory($destination_pre_path_2);
        } catch (\Exception $e) {

            $err = [
                'code' => 'ER0012',
                'message' => $e->getMessage(),
                'file' => $e->getFile(),
                'line' => $e->getLine()
            ];

            Log::info("WARNING :: ", $err);
        }

        foreach($get_uploaded_files as $val){
            $path = $val['path'] ;
            if(strlen($path) > 4 ){
                $src_path = storage_path()."/app/public/".$path ;
                File::copy($src_path, $destination_pre_path_2.'/'.$val['name'] ); // inside storage folder
                // Log::info("L0014 :: copying file as : ". $destination_pre_path_2.'/'.$val['name']) ;
            }
        }

        $folder_data = FolderDirectoryModel::select('section_id','folder_name','id')
            ->where('parent_folder_id', $folder_id )
            ->whereNull('deleted_at')
            ->get()->toArray() ;

        foreach($folder_data as $val){
            Log::info("L0012 :: need to download file for ") ;
            $this->downloadFolderRecursive( 
                [   'folder_id' => $val['id'], 
                    'bussiness_id' => $bsns_id,
                    'destination_pre_path' => $destination_pre_path_2.'/'
                ]);
        }
    }

}
