<?php

use Illuminate\Http\Request;
use App\Http\Controllers\UserController;
use App\Http\Controllers\GroupController;
use App\Http\Controllers\FileController;
use App\Http\Controllers\CollectionController;
use App\Http\Controllers\FolderDirectory;
use App\Http\Controllers\RoleController;
use App\Http\Controllers\SectionController;
use App\Http\Controllers\TemplateController;
use App\Http\Controllers\UserDirPermission;
use App\Http\Controllers\SuperadminController;
use App\Http\Controllers\AuditLogController;
use App\Http\Controllers\RecoveryController;
use App\Http\Controllers\DashboardController;
use App\Http\Controllers\NotificationController;
use App\Http\Controllers\DeleteManagementController;
use App\Http\Controllers\ReportsController;
use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/
Route::post('login',[UserController::class,'userLogin']);

Route::post('register-bussiness',[SuperadminController::class,'registerBussiness']);
Route::post('verify-registration-otp',[SuperadminController::class,'verifyOTP']);
Route::post('resend-otp',[SuperadminController::class,'resendOTP']);

Route::get('i_test', function(){     phpinfo();   });

// Route::get('send-mail', [UserController::class,'sendStaticMail']);

Route::middleware('auth:api')->group(function () {

    Route::post('upload-document',[CollectionController::class,'uploadDocument']);

    Route::post('add-user', [UserController::class,'addUser']);

    Route::get('collection-list/{user_id}/{folder_id}/{section_code}', [CollectionController::class,'collectionList']);

    Route::post('get-collection-by-search', [CollectionController::class,'searchInCollectionList']);
    Route::post('file-detail', [FileController::class,'fileDetail']);

    Route::post('create-folder', [FolderDirectory::class,'createFolder']);

    Route::get('del-file/{file_id}', [FileController::class,'delFile']);

    Route::get('del-folder/{folder_id}', [FolderDirectory::class,'delFolder']);

    Route::post('user-list',[UserController::class,'userList']);
    Route::post('user-detail',[UserController::class,'userDetail']);
    Route::post('user-delete',[UserController::class,'userDelete']);

    //section

    Route::post('section-add',[SectionController::class,'addSection']);
    Route::post('section-list',[SectionController::class,'sectionList']);
    Route::post('section-delete',[SectionController::class,'deleteSection']);
    Route::post('section-list-nav',[SectionController::class,'sectionListNav']);
    Route::post('section-detail',[SectionController::class,'sectionDetail']);

    Route::post('user-section-assoc',[SectionController::class,'getSectionAssocUser']);
    Route::post('save-user-section-assoc',[SectionController::class,'saveSectionAssocUser']);


    // group
    Route::post('group-add',[GroupController::class,'addGroup']);
    Route::post('group-list',[GroupController::class,'groupList']);
    Route::post('group-delete',[GroupController::class,'deleteGroup']);
    Route::post('user-group-assoc',[GroupController::class,'getGroupAssocUser']);
    Route::post('save-user-group-assoc',[GroupController::class,'saveGroupAssocUser']);
    // Route::post('group-detail',[GroupController::class,'groupDetail']);

    // template
    Route::post('template-add',[TemplateController::class,'addTemplate']);
    Route::post('template-list',[TemplateController::class,'templateList']);
    Route::post('template-delete',[TemplateController::class,'deleteTemplate']);
    Route::post('template-detail',[TemplateController::class,'templateDetail']);
    Route::post('template-list-for-assoc',[TemplateController::class,'templateListForAssoc']);
    Route::post('save-folder-template-assoc',[TemplateController::class,'saveTemplateFolderAssoc']);


    Route::post('update-file-detail', [FileController::class,'updateFileDetail']);
    Route::get('user-list-for-dir-permission', [UserDirPermission::class,'userListForDirPermission']);
    Route::post('get-user-permission', [UserDirPermission::class,'getUserPerm']);
    Route::post('save-user-permission', [UserDirPermission::class,'saveUserPerm']);
    // Route::post('user-template-assoc',[TemplateController::class,'templateList']);

    Route::get('get-user-detail', [UserController::class,'getUserDet']);

    Route::get('file-download/{file_id}/{is_uid}', [FileController::class,'downloadFile']);

    Route::post('save-link-document', [FileController::class,'saveLinkDocs']);

    Route::get('get-linked-document/{file_uid}', [FileController::class,'getLinkedDocs']);
    Route::get('del-linked-document/{file_uid}', [FileController::class,'delLinkedDocs']);

    Route::post('upload-version', [FileController::class,'uploadVersion']);
    Route::get('get-old-version/{file_uid}', [FileController::class,'getOldVersion']);
    Route::get('del-uploaded-version/{file_uid}', [FileController::class,'delOldVersion']);

    Route::post('update-user-profile', [UserController::class,'updateProfile']);

    Route::get('lock-document/{file_uid}', [FileController::class,'lockFile']);
    Route::get('unlock-document/{file_uid}', [FileController::class,'unlockFile']);

    Route::post('move-entity', [FolderDirectory::class,'moveEntity']);

    Route::post('get-global-audit-logs', [AuditLogController::class,'getLogs']);

    Route::get('get-registered-users', [SuperadminController::class,'getRegUsers']);

    Route::get('get-data-for-recycle', [RecoveryController::class,'getDeletedItems']);
    Route::post('recover-item', [RecoveryController::class,'recoverItem']);

    Route::get('get-dashboard-data', [DashboardController::class,'getDashboardData']); // for dashboard

    Route::get('download-folder/{folder_id}', [FolderDirectory::class,'downloadFolder']);

    Route::get('get-notification', [NotificationController::class,'getNotification']);

    Route::get('get-notification-all', [NotificationController::class,'getNotificationAll']);

    Route::post('del-multiple', [DeleteManagementController::class,'delMultiple']);

    Route::post('perm-del-multiple', [DeleteManagementController::class,'permDelMultiple']);
    Route::post('perm-del-single', [DeleteManagementController::class,'permDelSingle']);

    Route::get('isplanexpired', [CollectionController::class,'checkIfPlanExpired']);

    Route::get('get-folder-retention/{folder_id}', [FolderDirectory::class,'getFolderRetention']);
    Route::post('set-folder-retention', [FolderDirectory::class,'setFolderRetention']);

    Route::post('export-audit-log', [AuditLogController::class,'exportAuditLog']);
    Route::post('save-comment', [FileController::class,'saveDocumentComment']);
    Route::get('get-comment/{file_id}', [FileController::class,'getDocumentComment']);


    //Reports
    Route::get('get-version-details', [ReportsController::class, 'versionReportList']);

    Route::get('get-retention-details', [ReportsController::class, 'retentionReportList']);

    Route::get('exportRetention', [ReportsController::class,'exportRetentionReport']);

    Route::get('export-version-report', [ReportsController::class, 'exportVersionReport']);

    Route::post('get-template-details', [ReportsController::class, 'templateReportList']);

    Route::post('export-template-report', [ReportsController::class, 'exportTemplateReport']);

    Route::get('get-duplicate-report',[ReportsController::class, 'duplicateReport']);

    Route::get('export-duplicacy-report', [ReportsController::class, 'exportDuplicacyReport']);

});







